# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""Öğrenci Modülü

Bu modül Ulakbüs uygulaması için öğrenci modeli ve öğrenciyle ilişkili data modellerini içerir.

"""
import six

from pyoko import Model, field, ListNode, LinkProxy
from pyoko.exceptions import ObjectDoesNotExist
from pyoko.lib.utils import lazy_property
from .auth import Role, User
from .auth import Unit
from .buildings_rooms import Room, RoomType
from .personel import Personel


class HariciOkutman(Model):
    """Harici Okutman Modeli

    Harici okutmanın özlük ve iletişim bilgilerini içerir.

    """

    tckn = field.String("TC No", index=True)
    ad = field.String("Adı", index=True)
    soyad = field.String("Soyadı", index=True)
    cinsiyet = field.Integer("Cinsiyet", index=True, choices='cinsiyet')
    uyruk = field.String("Uyruk", index=True)
    medeni_hali = field.Integer("Medeni Hali", index=True, choices="medeni_hali", required=False)
    ikamet_adresi = field.String("İkamet Adresi", index=True, required=False)
    ikamet_il = field.String("İkamet İl", index=True, required=False)
    ikamet_ilce = field.String("İkamet İlçe", index=True, required=False)
    adres_2 = field.String("Adres 2", index=True, required=False)
    adres_2_posta_kodu = field.String("Adres 2 Posta Kodu", index=True, required=False)
    telefon_no = field.String("Telefon Numarası", index=True, required=True)
    oda_no = field.String("Oda Numarası", index=True, required=False)
    oda_tel_no = field.String("Oda Telefon Numarası", index=True, required=False)
    e_posta = field.String("E-Posta", index=True)
    e_posta_2 = field.String("E-Posta 2", index=True, required=False)
    e_posta_3 = field.String("E-Posta 3", index=True, required=False)
    web_sitesi = field.String("Web Sitesi", index=True, required=False)
    yayinlar = field.String("Yayınlar", index=True, required=False)
    projeler = field.String("Projeler", index=True, required=False)
    kan_grubu = field.String("Kan Grubu", index=True, required=False)
    ehliyet = field.String("Ehliyet", index=True, required=False)
    biyografi = field.Text("Biyografi")
    notlar = field.Text("Notlar")
    engelli_durumu = field.String("Engellilik", index=True)
    engel_grubu = field.String("Engel Grubu", index=True)
    engel_derecesi = field.String("Engel Derecesi")
    engel_orani = field.Integer("Engellilik Orani")
    cuzdan_seri = field.String("Seri", index=True)
    cuzdan_seri_no = field.String("Seri No", index=True)
    baba_adi = field.String("Ana Adi", index=True)
    ana_adi = field.String("Baba Adi", index=True)
    dogum_tarihi = field.Date("Doğum Tarihi", index=True, format="%d.%m.%Y")
    dogum_yeri = field.String("Doğum Yeri", index=True)
    kayitli_oldugu_il = field.String("İl", index=True)
    kayitli_oldugu_ilce = field.String("İlçe", index=True)
    kayitli_oldugu_mahalle_koy = field.String("Mahalle/Koy")
    kayitli_oldugu_cilt_no = field.String("Cilt No")
    kayitli_oldugu_aile_sira_no = field.String("Aile Sıra No")
    kayitli_oldugu_sira_no = field.String("Sıra No")
    kimlik_cuzdani_verildigi_yer = field.String("Cüzdanin Verildiği Yer")
    kimlik_cuzdani_verilis_nedeni = field.String("Cüzdanin Veriliş Nedeni")
    kimlik_cuzdani_kayit_no = field.String("Cüzdan Kayıt No")
    kimlik_cuzdani_verilis_tarihi = field.String("Cüzdan Kayıt Tarihi")
    akademik_yayinlari = field.String("Akademik Yayınları", index=True, required=False)
    verdigi_dersler = field.String("Verdiği Dersler", index=True, required=False)
    unvan = field.Integer("Unvan", index=True, choices="akademik_unvan", required=False)
    aktif = field.Boolean("Aktif", index=True, required=False)
    user = User(one_to_one=True)

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Harici Okutman"
        verbose_name_plural = "Harici Okutmanlar"
        search_fields = ['unvan', 'ad', 'soyad']

    def __unicode__(self):
        return '%s %s' % (self.ad, self.soyad)

    def post_save(self):
        """
        Okutman modelinden bu modele one-to-one bir bağlantı mevcuttur. Bu sebeple
        modelimizden de karşılıklı bir bağ oluşmuştur.

        Bu modelde yapılan değişiklik, bu metod sayesinde Okutman modeline yansıtılır.

        """

        if self.okutman.exist:
            self.okutman.ad = self.ad
            self.okutman.soyad = self.soyad
            self.okutman.unvan = self.unvan
            self.okutman.save()


class Okutman(Model):
    """Okutman Modeli

    Okutman bilgileri için data modelidir.

    Okutman, ders veren tüm öğretim elemanlarının (öğretim üyesi, öğretim elemanı,
    okutman ve harici okutman) genel adıdır.

    Bu model, aynı amaçla iki ayrı modele bağlıdır: Personel veya Harici Okutman.
    Ancak herbir kaydın sadece bir tanesine bağı olabilir. Okutman universite personeli
    veya dışarıdan harici bir personel olabilir. Aynı anda ikisi birden olamaz.

    Birim alanı okutmanın bağlı olduğu bölümü ifade eder. Harici okutmanlar da bu bölümlerin
    yetkisi altındadır.

    """

    ad = field.String("Ad", index=True, required=False)
    soyad = field.String("Soyad", index=True, required=False)
    unvan = field.String("Unvan", index=True, required=False)
    birim_no = field.String("Birim ID", index=True, required=False)
    personel = Personel(one_to_one=True)
    harici_okutman = HariciOkutman(one_to_one=True)

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Okutman"
        verbose_name_plural = "Okutmanlar"
        search_fields = ['unvan', 'personel']

    @lazy_property
    def okutman(self):
        """Okutmanın bağlı olduğu personel veya harici okutmanı döndürür.

        Example:
            Okutman örneği (instance) üzerinden erişilebilir. Aşağıdaki örnek ilgili okutmanın
            eposta bilgisine erişim sağlar. Okutman personel ise Personel modeli eposta alanı,
            harici okutman ise HariciOkutman modeli eposta alanı kullanılır::

                ders_okutman = Okutman.objects.get(ad="Yeter", soyad="Demir")
                eposta = ders_okutman.okutman.eposta

        """

        return self.personel if self.personel.exist else self.harici_okutman

    def __unicode__(self):
        return '%s %s' % (self.ad, self.soyad)

    def is_not_unique(self):
        """Personel veya Harici Okutman sayısını bulur.

        Returns:
            personel veya harici okutman sayısı

        """

        if self.personel.key:
            return len(self.objects.filter(personel=self.personel))
        elif self.harici_okutman.key:
            return len(self.objects.filter(harici_okutman=self.harici_okutman))

    def pre_save(self):
        """Bu metot nesne kayıt edilmeden hemen önce çalışır.

        Okutmanın bağlı olduğu personel veya harici okutman kaydı tekil olmalıdır. Bir okutmanın
        ya personel ya da harici okutman kaydı bulunabilir.

        Ayrıca aynı personel veya aynı harici okutmanın birden fazla Okutman kaydı bulunamaz.

        Bu metot ilgili kontrolleri yapıp, nesneyi kaydetmeye hazır hale getirir.


        Raises:
            Exception: Eğer kaydedilen nesne, veritabanında varsa ve tekil değilse


        """
        if self.okutman.key:
            self.ad = self.okutman.ad
            self.soyad = self.okutman.soyad
            self.unvan = self.okutman.unvan
        if not self.is_in_db() and self.is_not_unique():
            raise Exception("Okutman %s must be unique" % self.okutman)


class Donem(Model):
    """Dönem Modeli

    Güz, bahar ve yaz akademik dönemlerinin bilgilerine ait modeldir.

    Güncel alanı, içerisinde bulunulan akademik dönemi işaret eder.

    """

    ad = field.String("Ad", index=True)
    baslangic_tarihi = field.Date("Başlangıç Tarihi", index=True, format="%d.%m.%Y")
    bitis_tarihi = field.Date("Bitiş Tarihi", index=True, format="%d.%m.%Y")
    guncel = field.Boolean(index=True)

    @classmethod
    def guncel_donem(cls):
        return cls.objects.get(guncel=True)

    def pre_save(self):
        if self.guncel:
            try:
                old = self.guncel_donem()
                old.guncel = False
                old.save()
            except ObjectDoesNotExist:
                pass

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Dönem"
        verbose_name_plural = "Dönemler"
        list_fields = ['ad', 'baslangic_tarihi']
        search_fields = ['ad']

    def __unicode__(self):
        return '%s %s' % (self.ad, self.baslangic_tarihi)


class Program(Model):
    """Program Modeli

     Bir bölümün öğrenim programı (ders ve uygulamalardan oluşan) bilgilerinin
     saklandığı modeldir.

    """

    yoksis_no = field.String("YOKSIS ID", index=True)
    bolum_adi = field.String("Bölüm", index=True)
    ucret = field.Integer("Ücret", index=True)
    yil = field.String("Yıl", index=True)
    adi = field.String("Adı", index=True)
    tanim = field.String("Tanım", index=True)
    yeterlilik_kosullari_aciklamasi = field.String("Yeterlilik Koşulları Açıklaması", index=True)
    program_ciktilari = field.String("Program Çıktıları", index=True)
    mezuniyet_kosullari = field.String("Mezuniyet Koşulları", index=True)
    kabul_kosullari = field.String("Kabul Koşulları", index=True)
    donem_sayisi = field.Integer("Sürdüğü Dönem Sayısı", index=True)
    farkli_programdan_ders_secebilme = field.Boolean("Farklı Bir Programdan Ders Seçebilme",
                                                     default=False, index=True)
    bolum_baskani = Role(verbose_name='Bölüm Başkanı', reverse_name='bolum_baskani_program')
    ects_bolum_kordinator = Role(verbose_name='ECTS Bölüm Koordinator',
                                 reverse_name='ects_koordinator_program')
    akademik_kordinator = Role(verbose_name='Akademik Koordinator',
                               reverse_name='akademik_koordinator_program')
    birim = Unit(reverse_name="yoksis_program_program", verbose_name="YÖKSİS Program")

    bolum = Unit(reverse_name="bolum_program", verbose_name="Bölüm")

    # todo: to be removed
    # class Donemler(ListNode):
    #     donem = Donem()

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Program"
        verbose_name_plural = "Programlar"
        list_fields = ['adi', 'yil']
        search_fields = ['adi', 'yil', 'tanim']

    def __unicode__(self):
        return '%s %s' % (self.adi, self.yil)


class Ders(Model):
    """Ders Modeli

    Program dahilinde açılan derslerin bilgilerinin saklandığı modeldir.

    """

    ad = field.String("Ad", index=True)
    kod = field.String("Kod", index=True)
    tanim = field.String("Tanım", index=True)
    aciklama = field.String("Açıklama", index=True)
    onkosul = field.String("Önkoşul", index=True)
    uygulama_saati = field.Integer("Uygulama Saati", index=True)
    teori_saati = field.Integer("Teori Saati", index=True)
    ects_kredisi = field.Integer("ECTS Kredisi", index=True)
    yerel_kredisi = field.Integer("Yerel Kredisi", index=True)
    zorunlu = field.Boolean("Zorunlu", index=True)
    ders_dili = field.String("Ders Dili", index=True)
    ders_turu = field.Integer("Ders Türü", index=True, choices="ders_turleri")
    ders_amaci = field.String("Ders Amacı", index=True)
    ogrenme_ciktilari = field.String("Öğrenme Çıktıları", index=True)
    ders_icerigi = field.String("Ders İçeriği", index=True)
    ders_kategorisi = field.Integer("Ders Kategorisi", index=True, choices="ders_kategorileri")
    ders_kaynaklari = field.String("Ders Kaynakları", index=True)
    ders_mufredati = field.String("Ders Müfredatı", index=True)
    verilis_bicimi = field.Integer("Veriliş Biçimi", index=True, choices="ders_verilis_bicimleri")
    katilim_sarti = field.Integer("Katılım Şartı", index=True)
    ontanimli_kontenjan = field.Integer('Kontenjan', default=30)
    ontanimli_dis_kontenjan = field.Integer('Dış Kontenjan', default=5)
    program = Program()
    program_donemi = field.Integer('Programda Yeraldığı Dönem', index=True)
    donem = Donem()
    ders_koordinatoru = Personel()
    yerine_ders = LinkProxy("Ders", verbose_name="Yerine Açılan Ders", reverse_name="")

    class Degerlendirme(ListNode):
        tur = field.Integer("Değerlendirme Türü", choices="sinav_turleri", index=True)
        toplam_puana_etki_yuzdesi = field.Integer("Toplam Puana Etki Yüzdesi", index=True)

    class DersYardimcilari(ListNode):
        ders_yardimcilari = Personel()

    class DersVerenler(ListNode):
        dersi_verenler = Okutman()

    class DerslikTurleri(ListNode):
        """
        Bir dersin hangi derslik türlerinde kaç saat yapılacağı burada saklanır.
        Örneğin 5 saatlik bir dersin 3 saati sınıfta, 2 saati laboratuvarda
        yapılacaksa bu bilgi burada saklanır.
        """
        sinif_turu = RoomType()
        ders_saati = field.Integer("Ders Saati", index=True)

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Ders"
        verbose_name_plural = "Dersler"
        list_fields = ['ad', 'kod', 'ders_dili']
        search_fields = ['ad', 'kod']

    def __unicode__(self):
        return '%s %s %s' % (self.ad, self.kod, self.ders_dili)

    def ontanimli_sube_olustur(self):
        sube = Sube()
        sube.ad = 'Varsayılan Şube'
        sube.kontenjan = self.ontanimli_kontenjan
        sube.dis_kontenjan = self.ontanimli_dis_kontenjan
        sube.ders = self
        sube.donem = Donem.guncel_donem()
        sube.save()

    def pre_save(self):
        self.just_created = not self.exist

    def post_save(self):
        if self.just_created:
            self.ontanimli_sube_olustur()


class Sube(Model):
    """Şube Modeli

    Ders şubelerine ait bilgilerin saklandığı modeldir.

    Şube, bir dersin, bir dönem içerisinde okutmanı ile birlikte tanımlanmasıyla ortaya çıkar.

    Her şubenin önceden belirlenmiş bir kontenjanı vardır.

    Dış kontenjan ise, o şubeyi, dersin ait olduğu bölüm dışından seçebilecek
    öğrenci sayısını ifade eder.

    """

    ad = field.String("Ad", index=True)
    kontenjan = field.Integer("Kontenjan", index=True)
    dis_kontenjan = field.Integer("Dış Kontenjan", index=True)
    okutman = Okutman()
    ders = Ders()
    donem = Donem()
    ders_adi = field.String("Ders Adi", index=True)

    class NotDonusumTablosu(ListNode):
        """Not Donusum Tablosu

        Bu tablo, settings seklinde universite geneli icin tanimlanmistir.
        Eger okutman baska bir not donusum tablosu kullanmak isterse,
        harflendirme wf da bu tabloyu duzenlerse, tablo bir list node
        olarak bu modelde saklanir.

        """

        harf = field.String("Harf", index=True, choices="harf_notlari")
        dortluk_katsayi = field.Float("", choices="dortluk_katsayilari")
        yuzluk_not_baslangic = field.Float("Başlangıç", index=True)
        yuzluk_not_bitis = field.Float("Bitis", index=True)

        def __unicode__(self):
            return '%s %s %s %s' % (
                self.harf, self.yuzluk_not_baslangic, self.yuzluk_not_bitis, self.dortluk_katsayi)

    class Programlar(ListNode):
        programlar = Program()

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Şube"
        verbose_name_plural = "Şubeler"
        list_fields = ['ad', 'kontenjan']
        search_fields = ['ad', 'kontenjan']

    def sube_sinavlarini_olustur(self):
        for dg in self.ders.Degerlendirme:
            sinav = Sinav(tur=dg.tur, sube=self, ders=self.ders)
            sinav.save()

    def ders_adi_olustur(self):
        self.ders_adi = "%s - %s %s" % (self.ders.ad, self.ad, str(self.kontenjan))
        self.save()

    def post_creation(self):
        self.sube_sinavlarini_olustur()
        self.ders_adi_olustur()

    def __unicode__(self):
        return '%s' % self.ders_adi


class Sinav(Model):
    """Sınav Modeli

    Derse ait sınav(ara sınav, genel sınav, bütünleme, tek ders, muafiyet)
    bilgilerinin saklandığı modeldir.

    Sınavlar şubeler için ders dolayısı ile otomatik açılırlar. Bu sebeple temel bağ Şube
    modelidir.

    Ders arama kolaylığı için eklenmiştir.

    """

    tarih = field.Date("Sınav Tarihi", index=True)
    yapilacagi_yer = field.String("Yapılacağı Yer", index=True)
    tur = field.Integer("Sınav Türü", index=True, choices="sinav_turleri")
    aciklama = field.String("Açıklama", index=True)
    sube = Sube()
    degerlendirme = field.Boolean("Değerlendirme Durumu", index=True, default=False)

    # arama amacli
    ders = Ders()
    puan = field.Integer("Puan", index=True)

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Sınav"
        verbose_name_plural = "Sınavlar"
        list_fields = ['tarih', 'yapilacagi_yer']
        search_fields = ['aciklama', 'tarih']

    def __unicode__(self):
        return '%s %s %s' % (self.get_tur_display(), self.ders, self.sube)


class DersProgrami(Model):
    """Ders Programı Modeli

    Dersin işlenecegi gün, saat, şube ve derslik bilgilerini saklayan modeldir.

    """

    gun = field.String("Ders Günü", index=True)
    saat = field.Integer("Ders Saati", index=True)
    sube = Sube()
    derslik = Room()

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Ders Programı"
        verbose_name_plural = "Ders Programları"
        list_fields = ['gun', 'saat']
        search_fields = ['gun', 'saat']

    def __unicode__(self):
        return '%s %s' % (self.gun, self.saat)


class Ogrenci(Model):
    """Öğrenci Modeli

    Öğrencinin özlük ve iletişim bilgilerinin saklandığı modeldir.

    Öğrenciler, sisteme giriş yapar ve yetkileri doğrultusunda iş akışları
    çalıştırır. Bu amaçla User bağlantısı kurulmuştur.

    """

    ad = field.String("Ad", index=True)
    soyad = field.String("Soyad", index=True)
    cinsiyet = field.Integer("Cinsiyet", index=True, choices="cinsiyet")
    tckn = field.String("TC Kimlik No", index=True)
    cuzdan_seri = field.String("Seri", index=True)
    cuzdan_seri_no = field.String("Seri No", index=True)
    kayitli_oldugu_il = field.String("İl", index=True)
    kayitli_oldugu_ilce = field.String("İlçe", index=True)
    kayitli_oldugu_mahalle_koy = field.String("Mahalle/Köy")
    kayitli_oldugu_cilt_no = field.String("Cilt No")
    kayitli_oldugu_aile_sira_no = field.String("Aile Sıra No")
    kayitli_oldugu_sira_no = field.String("Sıra No")
    kimlik_cuzdani_verildigi_yer = field.String("Nüfus Cüzdanı Verildiği Yer")
    kimlik_cuzdani_verilis_nedeni = field.String("Nüfus Cüzdanı Veriliş Nedeni")
    kimlik_cuzdani_kayit_no = field.String("Nüfus Cüzdanı Kayıt No")
    kimlik_cuzdani_verilis_tarihi = field.Date("Nüfus Cüzdanı Veriliş Tarihi", index=True,
                                               format="%d.%m.%Y")
    baba_adi = field.String("Ana Adı", index=True)
    ana_adi = field.String("Baba Adı", index=True)
    ikamet_il = field.String("İkamet İl", index=True)
    ikamet_ilce = field.String("İkamet İlçe", index=True)
    ikamet_adresi = field.String("İkametgah Adresi", index=True)
    adres2 = field.String("2.Adres", index=True)
    posta_kodu = field.String("Posta Kodu", index=True)
    dogum_tarihi = field.Date("Doğum Tarihi", index=True, format="%d.%m.%Y")
    dogum_yeri = field.String("Doğum Yeri", index=True)
    uyruk = field.String("Uyruk", index=True)
    medeni_hali = field.Integer("Medeni Hali", index=True, choices="medeni_hali")
    ehliyet = field.String("Ehliyet", index=True)
    e_posta = field.String("E-Posta", index=True)
    e_posta2 = field.String("2.E-Posta", index=True)
    tel_no = field.String("Telefon Numarası", index=True)
    gsm = field.String("Cep Tel", index=True)
    kan_grubu = field.String("Kan Grubu", index=True)
    baba_aylik_kazanc = field.Integer("Babanızın Aylık Kazancı", index=True)
    baba_ogrenim_durumu = field.Integer("Babanızın Öğrenim Durumu", index=True,
                                        choices="ogrenim_durumu")
    baba_meslek = field.String("Babanızın Mesleği", index=True)
    anne_ogrenim_durumu = field.Integer("Annenizin Öğrenim Durumu", index=True,
                                        choices="ogrenim_durumu")
    anne_meslek = field.String("Annenizin Mesleği", index=True)
    anne_aylik_kazanc = field.Integer("Annenizin Aylık Kazancı", index=True)
    masraf_sponsor = field.Integer("Masraflarınız Kim Tarafından Karşılanacak", index=True,
                                   choices="masraf_sponsorlar")
    emeklilik_durumu = field.String("Velinizin Emeklilik Durumu", index=True)
    kiz_kardes_sayisi = field.Integer("Kız Kardeş Sayısı", index=True)
    erkek_kardes_sayisi = field.Integer("Erkek Kardeş Sayısı", index=True)
    ogrenim_goren_kardes_sayisi = field.Integer("Öğrenim Gören Kardeş Sayısı", index=True)
    burs_kredi_no = field.String("Kredi ve Yurtlar Kurumundan Aldığınız Kredi ve Burs No",
                                 index=True)
    aile_tel = field.String("Ailenizin Ev Tel", index=True)
    aile_gsm = field.String("Ailenizin Cep Tel", index=True)
    aile_adres = field.String("Ailenizin Daimi İkamet Ettiği Adres", index=True)
    ozur_durumu = field.Integer("Varsa Özür Durumunuz", index=True, choices="ozur_durumu")
    ozur_oran = field.Integer("Varsa Özür Oranınız %", index=True)
    user = User(one_to_one=True)

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Ögrenci"
        verbose_name_plural = "Ögrenciler"
        list_fields = ['ad', 'soyad']
        search_fields = ['ad', 'soyad']

    def __unicode__(self):
        return '%s %s' % (self.ad, self.soyad)


class OncekiEgitimBilgisi(Model):
    """Öncekı Eğitim Bilgisi Modeli

    Öğrenciye ait önceki eğitim bilgisi modelidir.

    """

    okul_adi = field.String("Mezun Olduğu Okul", index=True)
    diploma_notu = field.Float("Diploma Notu", index=True)
    mezuniyet_yili = field.String("Mezuniyet Yılı", index=True)
    ogrenci = Ogrenci()

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Önceki Eğitim Bilgisi"
        verbose_name_plural = "Önceki Eğitim Bilgileri"
        list_fields = ['okul_adi', 'diploma_notu', 'mezuniyet_yili']
        search_fields = ['okul_adi', 'diploma_notu', 'mezuniyet_yili']

    def __unicode__(self):
        return '%s %s %s' % (self.okul_adi, self.mezuniyet_yili, self.ogrenci.ad)


class OgrenciProgram(Model):
    """Öğrenci Program Modeli

    Öğrencilerin kayıt yaptırdığı programların saklandığı modeldir.

    Öğrenciler birden fazla programa kayıt yaptırabilirler. Herbir program için ayrı bir öğrenci
    numarası alırlar.

    Aktif dönem öğrencinin ilgili programda geldiği aşamayı ifade eder. Buna göre ilgili program
    derslerinden faydalanabilir.

    Başarı durumu, Genel Ağırlıklı Not Ortalamasını ifade eder:
    http://www.ulakbus.org/wiki/standart_fakulteler_icin_yazilim_ihtiyac_analizi_belgesi.html#basari-hesaplama

    Öğrencinin ilgili programdaki danışman bilgisi de burada saklanır.

    """

    ogrenci_no = field.String("Öğrenci Numarası", index=True)
    giris_tarihi = field.Date("Giriş Tarihi", index=True, format="%d.%m.%Y")
    mezuniyet_tarihi = field.Date("Mezuniyet Tarihi", index=True, format="%d.%m.%Y")
    giris_puan_turu = field.Integer("Puan Türü", index=True, choices="giris_puan_turleri")
    giris_puani = field.Float("Giriş Puani", index=True)
    aktif_donem = field.String("Dönem", index=True)
    ogrencilik_statusu = field.Integer('Öğrencilik Statüsü', index=True,
                                       choices="ogrenci_program_durumlar")
    ogrenci_ders_kayit_status = field.Integer('Öğrencilik Ders Kayıt Statüsü', index=True,
                                              choices="ogrenci_ders_kayit_durum")
    ayrilma_nedeni = field.Integer('Ayrılma Nedeni', index=True, choices='ayrilma_nedeni')
    basari_durumu = field.String("Başarı Durumu", index=True)
    diploma_no = field.String("Diploma No", index=True)
    ders_programi = DersProgrami()
    danisman = Personel()
    program = Program()
    ogrenci = Ogrenci()

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Öğrenci Programı"
        verbose_name_plural = "Öğrenci Programları"

    class Belgeler(ListNode):
        tip = field.Integer("Belge Tipi", choices="belge_tip", index=True)
        aciklama = field.String("Ek Açıklama", index=True, default="-", required=False)
        tamam = field.Boolean("Belge kontrol edildi", index=True, required=True)

    class OgrenciDonem(ListNode):
        donem = Donem()

    def __unicode__(self):
        return '%s %s - %s / %s' % (self.ogrenci.ad, self.ogrenci.soyad,
                                    self.program.adi, self.program.yil)

    def tarih_sirasiyla_donemler(self):
        r = []
        for ogd in self.OgrenciDonem:
            r.append((ogd.donem.ad, ogd.donem.baslangic_tarihi))
        r.sort(key=lambda tup: tup[1])
        return r


class OgrenciDersi(Model):
    """Öğrenci Dersi Modeli

    Öğrencilerin ders seçimlerinin saklandığı modeldir.

    Ders alanı Şube modeli ile ilişkilendirilmiştir. Bunun sebebi öğrencilerin ders seçiminin,
    ders ve okutmanın birleştiği şube seçimi olmasıdır. Detaylı bilgiler Şube modelinde bulunabilir.

    Bir öğrencinin devamsızlıktan kalıp kalmadığı devamsizliktan_kalma alanı ile kontrol edilir.
    Bu alan False olduğu zaman öğrenci devamsızlıktan kalır.

    """

    alis_bicimi = field.Integer("Dersi Alış Biçimi", index=True)
    sube = Sube(unique=True)
    ogrenci_program = OgrenciProgram()
    ogrenci = Ogrenci(unique=True)
    basari_ortalamasi = field.Float("Ortalama", index=True)
    harflendirilmis_not = field.String("Harf", index=True)
    katilim_durumu = field.Boolean("Devamsızlıktan Kalma", default=False, index=True)

    # arama amaçlı alanlar
    ders = Ders()
    donem = Donem()

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Ögrenci Dersi"
        verbose_name_plural = "Öğrenci Dersleri"
        list_fields = ['ders', 'alis_bicimi']
        search_fields = ['alis_bicimi', ]
        unique_together = [('ogrenci', 'sube')]

    def post_creation(self):

        """
        Yeni bir ``OgrenciDers``'i ilk defa yaratılınca ``donem`` ve ``ders`` alanları,
        bağlı şubeden atanır.

        """

        self.donem = self.sube.donem
        self.ders = self.sube.ders
        self.save()

    def ders_adi(self):
        """
        Şubenin bağlı olduğu ders adı.

        Returns:
            Şubenin bağlı olduğu ders örneğinin adını döndürür.

        """

        return six.text_type(self.ders.ad)

    ders_adi.title = 'Ders'

    def __unicode__(self):
        return '%s %s %s' % (self.ders.kod, self.ders.ad, self.alis_bicimi)


class DersKatilimi(Model):
    """Ders Katılımı Modeli

    Öğrencilerin devam durumları hakkında bilgilerin saklandığı modeldir. Okutman tarafından
    verilecek yüzdelik bir ifadeyle katılım durumu alanında saklanır.

    Temel ilişki Ogrenci ve Ders modelleri ile kurulmuştur.

    Note:
        Okutman arama kolaylığı amacıyla saklanmıştır.

    """

    # TODO: Neden float, soralım?
    katilim_durumu = field.Integer("Katılım Durumu", index=True)
    ders = Sube()
    ogrenci = Ogrenci()
    okutman = Okutman()

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Ders Devamsızlığı"
        verbose_name_plural = "Ders Devamsızlıklari"
        list_fields = ['katilim_durumu', 'sube_dersi']
        search_fields = ['sube_dersi', 'katilim_durumu']

    def sube_dersi(self):
        """
        Şubenin bağlı olduğu ders adı.

        Returns:
            Şubenin bağlı olduğu ders örneğinin adını döndürür.

        """

        return six.text_type(self.ders.ders)

    sube_dersi.title = 'Ders'

    def __unicode__(self):
        return '%s %s' % (self.katilim_durumu, self.ogrenci)


class Borc(Model):
    """Borç modeli

    Öğrencilerin ödemesi gereken ücret (harc, belge, belgeler, kimlik
    ücretleri vb.) bilgilerinin saklandığı modeldir.

    ``tahakkuk_referans_no`` sistem tarafından üretilen ve
    3. taraflara (banka veya ilgili diğer kurumlar) iletilen tekil
    takip koddur.

    """

    miktar = field.Float("Borç Miktarı", index=True)
    para_birimi = field.Integer("Para Birimi", index=True, choices="para_birimleri")
    sebep = field.Integer("Borç Sebebi", index=True, choices="ogrenci_borc_sebepleri")
    son_odeme_tarihi = field.Date("Son Ödeme Tarihi", index=True)
    tahakkuk_referans_no = field.String("Tahakkuk Referans No", index=True)
    aciklama = field.String("Borç Açıklaması", index=True)
    ogrenci = Ogrenci()
    donem = Donem()

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Borç"
        verbose_name_plural = "Borçlar"
        list_fields = ['miktar', 'son_odeme_tarihi']
        search_fields = ['miktar', 'odeme_tarihi']

    def __unicode__(self):
        return '%s %s %s %s' % (self.miktar, self.para_birimi, self.sebep, self.son_odeme_tarihi)


class Banka(Model):
    """Banka Modeli

    Basitçe banka adı ve tekil bir banka kodu ile saklanır.

    """

    ad = field.String("Banka Adi", index=True)
    kod = field.String("Banka Kodu", index=True)

    class Meta:
        verbose_name = "Banka"
        verbose_name_plural = "Bankalar"

    def __unicode__(self):
        return '%s %s' % (self.ad, self.kod)


class Odeme(Model):
    """Ödeme Modeli

    Öğrencilerin borçlarına karşılık, banka veya diğer yollar ile tahsil
    edilen ödemelerin saklandığı data modelidir.

    """

    miktar = field.Float("Borç Miktarı", index=True)
    para_birimi = field.Integer("Para Birimi", index=True, choices="para_birimleri")
    aciklama = field.String("Borç Açıklaması", index=True)
    odeme_sekli = field.Integer("Ödeme Şekli", index=True, choices="odeme_sekli")
    odeme_tarihi = field.Date("Ödeme Tarihi", index=True)
    borc = Borc()
    ogrenci = Ogrenci()
    banka = Banka()
    banka_sube_kodu = field.String("Banka Sube Kodu", index=True)
    banka_kanal_kodu = field.String("Kanal Kodu", index=True)
    tahsilat_referans_no = field.String("Tahsilat Referans No", index=True)
    donem = Donem()

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Borc"
        verbose_name_plural = "Borclar"
        list_fields = ['miktar', 'son_odeme_tarihi']
        search_fields = ['miktar', 'odeme_tarihi']

    def __unicode__(self):
        return '%s %s %s %s' % (self.miktar, self.para_birimi, self.sebep, self.son_odeme_tarihi)


class BankaAuth(Model):
    """Banka Doğrulama Modeli

    Banka kullanıcılarının doğrulanması için gereken bilgilerin
    tutulduğu data modelidir.

    """

    username = field.String("Username", index=True)
    password = field.String("Password", index=True)
    banka = Banka()

    class Meta:
        verbose_name = "Banka Kullanicisi"
        verbose_name_plural = "Banka Kullanicilari"

    def __unicode__(self):
        return '%s %s' % (self.username, self.banka.ad)


class DegerlendirmeNot(Model):
    """Değerlendirme Notu Modeli

    Ders değerlendirmeleri (sınavlar, sunum, proje, odev vb.) için okutmanlar
    tarafından verilen notların saklandığı data modelidir.

    Temel ilişki Sınav ve Öğrenci modeli ile kurulmuştur. Değerlendirme
    bilgisi puan alanında saklanır.

    Note:
        Ders, öğretim elemeanı, yıl ve donem, alanları arama kolaylığı
        açısından saklanmaktadır.

    """

    puan = field.Integer("Puan", index=True)
    sinav = Sinav()
    ogrenci = Ogrenci()
    aciklama = field.String("Puan Açıklaması", index=True, required=False)

    # Arama amacli alanlar.
    yil = field.String("Yıl", index=True)
    donem = field.String("Dönem", index=True)
    ogretim_elemani = field.String("Öğretim Elemanı", index=True)
    ogrenci_no = field.String("Öğrenci No", index=True)
    sinav_tarihi = field.Date("Sınav Tarihi", index=True)
    ders = Ders()

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Not"
        verbose_name_plural = "Notlar"
        list_fields = ['puan', 'ders_adi']
        search_fields = ['aciklama', 'puan', 'ogrenci_no']
        list_filters = ['donem', ]

    def post_save(self):
        self.sinav.degerlendirme = True
        self.sinav.puan = self.puan
        self.sinav.save()

    def ders_adi(self):
        return "%s" % self.ders.ad

    ders_adi.title = "Ders"

    def __unicode__(self):
        return '%s %s' % (self.puan, self.sinav)


AKADEMIK_TAKVIM_ETKINLIKLERI = [
    ('1', 'Yeni Öğrenci Ön Kayıt'),
    ('2', 'Güz Dönem Başlangıcı'),
    ('3', 'Derslerin Acılması'),
    ('4', 'Subelendirme ve Ders Programının Ilan Edilmesi'),
    ('5', 'Öğrenci Harç'),
    ('6', 'Öğrenci Ek Harç'),
    ('7', 'Mazeretli Öğrenci Harç'),
    ('8', 'Yeni Öğrenci Ders Kayıt'),
    ('9', 'Yeni Öğrenci Danışman Onay'),
    ('10', 'Ders Kayıt'),
    ('11', 'Danışman Onay'),
    ('12', 'Mazeretli Ders Kayıt'),
    ('13', 'Mazeretli Danışman Onay'),
    ('14', 'Ders Ekle/Bırak'),
    ('15', 'Ders Ekle/Bırak Danışman Onay'),
    ('16', 'Danışman Dersten Çekilme İşlemleri'),
    ('17', 'Ara Sinav'),
    ('18', 'Ara Sınav Not Giriş'),
    ('19', 'Ara Sınav Notlarının Öğrenciye Yayınlanması'),
    ('20', 'Ara Sinav Mazeretli'),
    ('21', 'Ara Sınav Mazeret Not Giriş'),
    ('22', 'Ara Sınav Mazeret Notlarının Öğrenciye Yayınlanması'),
    ('23', 'Sınav Maddi Hata Düzeltme'),
    ('24', 'Yariyil Sinav'),
    ('25', 'Yarıyıl Sınavı Not Giriş'),
    ('26', 'Yarıyıl Sınavı Notlarının Öğrenciye Yayınlanmasi'),
    ('27', 'Bütünleme ve Yarı Yıl Sonu Mazeret Sınavı'),
    ('28', 'Bütünleme ve Yarı Yıl Sonu Mazeret Sınavı Not Giriş'),
    ('29', 'Bütünleme ve Yarı Yıl Sonu Mazeret Sınavı Notlarının Öğrenciye Yayınlanması'),
    ('30', 'Harf Notlarının Öğrenciye Yayınlanması'),
    ('31', 'Bütünleme Harf Notlarının Öğrenciye Yayınlanması'),
    ('32', 'Öğretim Elemanı Yoklama Girişi'),
    ('33', 'Bahar Donemi Derslerin Acilmasi'),
    ('34', 'Bahar Donemi Subelendirme ve Ders Programının Ilan Edilmesi'),
    ('35', 'Bahar Dönem Başlangıcı'),
    ('36', 'Öğrenci Harç'),
    ('37', 'Öğrenci Ek Harç'),
    ('38', 'Mazeretli Öğrenci Harç'),
    ('39', 'Ders Kayıt'),
    ('40', 'Danışman Onay'),
    ('41', 'Mazeretli Ders Kayıt'),
    ('42', 'Mazeretli Danışman Onay'),
    ('43', 'Ders Ekle / Bırak'),
    ('44', 'Ders Ekle / Bırak Onay'),
    ('45', 'Ara Sinav'),
    ('46', 'Ara Sınav Not Giriş'),
    ('47', 'Ara Sınav Notlarının Öğrenciye Yayınlanması'),
    ('48', 'Ara Sinav Mazeretli'),
    ('49', 'Ara Sınav Mazeret Not Giriş'),
    ('50', 'Ara Sınav Mazeret Notlarının Öğrenciye Yayınlanması'),
    ('51', 'Sınav Maddi Hata Düzeltme'),
    ('52', 'Yariyil Sinav'),
    ('53', 'Yarıyıl Sınavı Not Giriş'),
    ('54', 'Yarıyıl Sınavı Notlarının Öğrenciye Yayınlanmasi'),
    ('55', 'Öğretim Elemanı Yoklama Girişi'),
]


class AkademikTakvim(Model):
    """Akademik Takvim Modeli

    Akademik Takvim etkinlikleri bilgileri modeldir. AKADEMIK_TAKVIM_ETKINLIKLERI ile
    tanımlanmış her bir etkinlik için tarih veya tarih aralığı bigisi saklanır.

    Universiteye ait bir takvim zorunlu olarak varolmakla birlikte
    istenirse etkinlikler birimlere göre farklılık gösterebilirler.

    """

    birim = Unit("Birim", index=True)
    yil = field.Date("Yıl", index=True)

    class Takvim(ListNode):
        etkinlik = field.Integer("Etkinlik", index=True, choices=AKADEMIK_TAKVIM_ETKINLIKLERI)
        baslangic = field.Date("Başlangıç", index=True, format="%d.%m.%Y")
        bitis = field.Date("Bitiş", index=True, format="%d.%m.%Y", required=False)

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Akademik Takvim"
        verbose_name_plural = "Akademik Takvimler"
        list_fields = ['_birim', 'yil']
        # search_fields = ['yil']

    def _birim(self):
        return "%s" % self.birim

    _birim.title = 'Birim'

    def __unicode__(self):
        return '%s %s' % (self.birim, self.yil)


class DonemDanisman(Model):
    """Dönem Danışmanları Modeli

    Dönem, Bölüm ve Program bazlı olarak öğrencilere danışman
    atanabilecek olan öğretim elemanlarının saklandığı data modelidir.

    """

    donem = Donem()
    okutman = Okutman()
    bolum = Unit()
    aciklama = field.String("Açıklama", index=True, required=False)

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Dönem Danışman"
        verbose_name_plural = "Dönem Danışmanları"
        list_fields = ['program', 'okutman', 'bolum', 'donem']
        search_fields = ['aciklama']

    def __unicode__(self):
        return '%s %s' % (self.bolum, self.okutman)


class DondurulmusKayit(Model):
    """Dondurulmuş Kayıt Modeli

    Dondurulmuş öğrenci kayıtlarının saklandığı data modelidir.

    """

    donem = Donem()
    ogrenci_program = OgrenciProgram()
    baslangic_tarihi = field.Date("Başlangıç Tarihi", index=True)
    aciklama = field.String("Açıklama", index=True, required=False)

    class Meta:
        app = 'Ogrenci'
        verbose_name = "Dondurulmuş Kayıt"
        verbose_name_plural = "Dondurulmuş Kayıtlar"
        list_fields = ['ogrenci_program', 'baslangic_tarihi', 'aciklama', 'donem']
        search_fields = ['aciklama', 'baslangic_tarihi']

    def __unicode__(self):
        return '%s %s' % (self.ogrenci_program, self.donem)



# -*-  coding: utf-8 -*-
"""
"""


# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

def prepare_choices_for_model(model, **kwargs):
    """Model için Seçenekler Hazırla

    Args:
        model: Model
        **kwargs: Keyword argümanları

    Returns:
        Keyword argümanlara göre filtrelenmiş modelin,
        key ve __unicode__ method değerlerini

    """

    return [(m.key, m.__unicode__()) for m in model.objects.filter(**kwargs)]


def prepare_titlemap_for_model(model, **kwargs):
    """Model için Typeahead Seçenekleri Hazırla

    Args:
        model: Model
        **kwargs: Keyword argümanları

    Returns:
        Keyword argümanlara göre filtrelenmiş modelin,
        key ve __unicode__ method değerlerini

    """

    return [(m.key, m.__unicode__()) for m in model.objects.filter(**kwargs)]



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP İstisnai İlgi Sorgula

Hitap üzerinden personelin istisnai ilgi bilgilerinin sorgulamasını yapar.

Note:
    Bu servis, service ve bean isimlerindeki hatadan dolayı çalışmamaktadır.
    Açıklama için ilgili birimlere başvuruldu, yanıt bekleniyor.

"""

from ulakbus.services.personel.hitap.hitap_sorgula import HITAPSorgula


class HizmetIstisnaiIlgiGetir(HITAPSorgula):
    """
    HITAP Sorgulama servisinden kalıtılmış
    İstisnai İlgi Bilgisi Sorgulama servisi

    """

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            bean_name (str): Hitap'tan gelen bean nesnesinin adı
            service_dict (dict): Hitap servisinden gelen kayıtların alanları,
                    ``HizmetIstisnaiIlgi`` modelinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları ve servis tarafında gerekli olan
                    alanlar listede tutulmaktadır.

        """

        self.service_name = 'hizmetIstisnaiIlgiSorgu'
        self.bean_name = 'HizmetIstisnaiIlgiServisBean'
        self.service_dict = {
            'fields': {
                'tckn': 'tckn',
                'kayit_no': 'kayitNo',
                'baslama_tarihi': 'baslamaTarihi',
                'bitis_tarihi': 'bitisTarihi',
                'gun_sayisi': 'gunSayisi',
                'istisnai_ilgi_nevi': 'istisnaiIlgiNevi',
                'kha_durum': 'khaDurum',
                'kurum_onay_tarihi': 'kurumOnayTarihi'
            },
            'date_filter': ['baslama_tarihi', 'bitis_tarihi', 'kurum_onay_tarihi'],
            'required_fields': ['tckn']
        }
        super(HizmetIstisnaiIlgiGetir, self).handle()

    def custom_filter(self, hitap_dict):
        """
        Hitap sözlüğüne uygulanacak ek filtreleri gerçekleştirir.

        Args:
            hitap_dict (List[dict]): Hitap verisini yerele uygun biçimde tutan sözlük listesi

        """

        for record in hitap_dict:
            record['kha_durum'] = self.kha_durum_kontrol(record['kha_durum'])

    def kha_durum_kontrol(self, kha_durum):
        """
        Hitap İstisnai İlgi servisinin,
        "0" veya "1" olarak gelen Kazanılmış Hak Aylığı durum bilgisi değeri,
        tam sayı olarak elde edilmektedir.

        - "0" : "Değerlendirilmedi"
        - "1" : "Değerlendirildi"

        Args:
            kha_durum (str): İstisnai İlgi KHA durum değeri.

        Returns:
            int: KHA durum tam sayı değeri.

        Raises:
            ValueError: Geçersiz KHA durum kodu.

        """

        try:
            return int(kha_durum)
        except ValueError:
            self.logger.exception("KHA Durum kodu gecersiz: %s" % kha_durum)
            return 0



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
from ulakbus.models import OgrenciProgram, OgrenciDersi, Sinav, DegerlendirmeNot, Ogrenci
from ulakbus.views.ders.ders import prepare_choices_for_model
from zengine import forms
from zengine.forms import fields
from zengine.views.crud import CrudView


class NotDuzenlemeForm(forms.JsonForm):
    """
    `NotDuzenleme`` sınıfı için form olarak kullanılacaktır. Form,
    include listesinde, aşağıda tanımlı alanlara sahiptir.

    """

    class Meta:
        include = ['puan']

    kaydet = fields.Button('Kaydet')


class NotDuzenleme(CrudView):
    """Öğrenci İşleri Not Düzenleme İş Akışı

   Öğrenci İşleri Not Düzenleme  iş akışı 6 adımdan oluşmaktadır.

   * Fakülte Yönetim Karar No
   * Öğrenci Program Seç
   * Ders Seç
   * Sınav Seç
   * Not Düzenle
   * Bilgilendir

   Bu iş akışında kullanılan metotlar şu şekildedir.

   Fakülte Yönetim Karar No:
     Fakülte Yönetim Kurulu tarafından belirlenen karar no girilir.

   Program Seç:
      Öğrencinin kayıtlı olduğu öğrenci program seçilir.

    Ders Seç:
      Öğrencinin kayıtlı olduğu derslerden biri seçilir

    Sınav Seç:
      Derse ait sınavlardan biri seçilir.

    Not Düzenle:
      Seçilen sınavın puanı düzenlenir.

    Bilgilendir:
      Not düzenleme işlemi tamamlandıktan sonra ekrana bilgilendirme mesajı basılır.

    Bu sınıf ``CrudView`` extend edilerek hazırlanmıştır. Temel model ``DeğerlendirmeNot``
    modelidir. Meta.model bu amaçla kullanılmıştır.

    Adımlar arası geçiş manuel yürütülmektedir.

    """

    class Meta:
        model = 'DegerlendirmeNot'

    def fakulte_yonetim_karari(self):
        """
        Fakülte Yönetim Kurulu tarafından belirlenen karar no girilir.

        """

        # TODO: Fakülte yönetim kurulunun kararı loglanacak.
        self.current.task_data['ogrenci_id'] = self.current.input['id']
        _form = forms.JsonForm(current=self.current,
                               title='Fakülte Yönetim Kurulunun Karar Numarasını Giriniz.')
        _form.karar = fields.String('Karar No', index=True)
        _form.kaydet = fields.Button('Kaydet')
        self.form_out(_form)

    def program_sec(self):
        """
        Öğrencinin kayıtlı olduğu öğrenci program seçilir.

        """

        _form = forms.JsonForm(current=self.current, title='Program Seçiniz.')
        _choices = prepare_choices_for_model(OgrenciProgram,
                                             ogrenci_id=self.current.task_data['ogrenci_id'])
        _form.program = fields.Integer(choices=_choices)
        _form.onayla = fields.Button('Seç')
        self.form_out(_form)

    def ders_sec(self):
        """
        Öğrencinin kayıtlı olduğu derslerden biri seçilir.

        """

        program_id = self.current.input['form']['program']
        _form = forms.JsonForm(current=self.current, title='Ders Seçiniz.')
        _choices = prepare_choices_for_model(OgrenciDersi, ogrenci_program_id=program_id)
        _form.ders = fields.Integer(choices=_choices)
        _form.onayla = fields.Button('Seç')
        self.form_out(_form)

    def sinav_sec(self):
        """
        Derse ait sınavlardan biri seçilir.

        """

        self.current.task_data['ders_id'] = self.current.input['form']['ders']
        ogrenci_dersi = OgrenciDersi.objects.get(self.current.task_data['ders_id'])

        _form = forms.JsonForm(current=self.current, title='Sınav Seçiniz.')
        _choices = prepare_choices_for_model(Sinav, sube_id=ogrenci_dersi.sube.key)
        _form.sinav = fields.Integer(choices=_choices)
        _form.onayla = fields.Button('Seç')
        self.form_out(_form)

    def not_duzenle(self):
        """
        Seçilen sınavın puanı düzenlenir.

        """

        sinav_id = self.current.input['form']['sinav']
        sinav = Sinav.objects.get(sinav_id)
        self.current.task_data['sinav'] = sinav.__unicode__()
        ogrenci_id = self.current.task_data['ogrenci_id']
        degerlendirme_not = \
            DegerlendirmeNot.objects.get(sinav_id=sinav_id, ogrenci_id=ogrenci_id)
        self.current.task_data['onceki_puan'] = degerlendirme_not.puan

        title = '%s adlı öğrencinin % sınava ait notunu düzenleyiniz.' % (
            degerlendirme_not.ogrenci, degerlendirme_not.sinav)
        _form = NotDuzenlemeForm(degerlendirme_not, current=self.current, title=title)

        self.form_out(_form)

    def bilgilendir(self):
        """
        Not düzenleme işlemi tamamlandıktan sonra ekrana bilgilendirme mesajı basılır.

        """

        ogrenci_id = self.current.task_data['ogrenci_id']
        ogrenci = Ogrenci.objects.get(ogrenci_id)
        yeni_puan = self.current.input['form']['puan']
        sinav = self.current.task_data['sinav']
        onceki_puan = self.current.task_data['onceki_puan']

        self.current.output['msgbox'] = {
            'type': 'info', "title": 'Not Düzeltme Tamamlandı',
            "msg": '%s adlı öğrencinin, %s sınavına ait %s olan notu, %s ile değiştilmiştir.' % (
                ogrenci, sinav, onceki_puan, yeni_puan)
        }



# -*- coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.


from zato.server.service import Service
from ulakbus import settings

__author__ = 'Ali Riza Keles'

UID = settings.UID

DEBUG = False
if DEBUG:
    import logging

    logging.basicConfig(level=logging.INFO)
    logging.getLogger('suds.client').setLevel(logging.DEBUG)
    logging.getLogger('suds.transport').setLevel(logging.DEBUG)
    logging.getLogger('suds.xsd.schema').setLevel(logging.DEBUG)
    logging.getLogger('suds.wsdl').setLevel(logging.DEBUG)


class YOKSIS(Service):
    def __init__(self):
        super(YOKSIS, self).__init__()
        self.cli = self.connection()
        self.birim = 0

    @staticmethod
    def connection():
        """
        Bu metod Zato suds proxy kullanimi icin patch yapilana kadar burada duracak.
        :return: suds client
        """

        # conn = self.outgoing.soap['YOKSIS Akademik Birim Agaci'].conn
        # cli = conn.client()

        from suds.client import Client
        from suds.cache import NoCache
        wsdl = 'http://servisler.yok.gov.tr/ws/UniversiteBirimlerv1?WSDL'
        proxy = {'http': 'services.konya.edu.tr:3128'}
        cli = Client(wsdl, cache=NoCache(), proxy=proxy)
        cli.set_options(faults=False)
        return cli

    def birim_detaylari(self):
        birim = self.get_birim_from_returned_tuple()
        ret = {
            'aktif': birim.AKTIF,
            'birim_adi': birim.BIRIM_ADI,
            'birim_id': birim.BIRIM_ID,
            'birim_uzun_adi': birim.BIRIM_UZUN_ADI,
            'il_kodu': birim.IL_KODU,
            'ogrenim_turu': birim.OGRENIM_TURU,
            'bagli_oldugu_birim_id': birim.BAGLI_OLDUGU_BIRIM_ID,
            'birim_turu_adi': birim.BIRIM_TURU_ADI,
            'ilce_kodu': birim.ILCE_KODU,
            'klavuz_kodu': birim.KILAVUZ_KODU,
            'ogrenim_suresi': birim.OGRENIM_SURESI,
            'ogrenim_dili': '',
            'birim_adi_ingilizce': ''
        }
        if birim.OGRENIM_DILI:
            ret.update({'ogrenim_dili': birim.OGRENIM_DILI})
        if birim.BIRIM_ADI_INGILIZCE:
            ret.update({'birim_adi_ingilizce': birim.BIRIM_ADI_INGILIZCE})

        return ret

    def birim_detaylari_utf8(self):
        detaylar = self.birim_detaylari()
        for k, v in detaylar:
            if v is not '' and type(v) is 'str':
                detaylar[k] = v.encode('utf-8')
        return detaylar

    def birim_kaydet(self, birim_id):
        # self.kvdb.conn.set(birim_id, self.birim_detaylari())
        # self.logger.info("%s icin degerler: %s\n\n" % (birim_id, self.birim_detaylari()))
        from pyoko.db.connection import client
        yoksis_birim = client.bucket_type('catalog').bucket('ulakbus_yoksis_birim')

        y = yoksis_birim.get(str(birim_id))
        data = self.birim_detaylari()
        y.data = data
        # print("%s: stored.." % (birim_id))
        self.logger.info("%s icin kaydedildi: \n\n" % birim_id)
        y.store()

    def get_birim_from_returned_tuple(self):
        return self.birim[1][0]


class BirimDetay(YOKSIS):
    """
    YOKSIS Birim Detay Zato Servisi
    """

    def handle(self):
        birim_id = self.request.payload['birim_id']
        self.birim = self.cli.service.IDdenBirimAdiGetir(birim_id)
        return self.birim_detaylari()


class BirimAgaci(YOKSIS):
    """
    YOKSIS BirimAgaci Kaydet Servisi
    """

    def bir(self, conn, root_unit):
        # walk through alt birimler
        # start from 0
        self.birim = conn.service.IDdenBirimAdiGetir(root_unit)
        if root_unit != 0:
            self.birim_kaydet(birim_id=root_unit)

        alt_birimler = conn.service.AltBirimleriGetir(root_unit)
        alt_birimler = alt_birimler[1].Birimler
        if len(alt_birimler) > 0:
            for b in alt_birimler:
                if b is None:
                    continue
                else:
                    self.bir(conn, root_unit=b.BIRIM_ID)

    def handle(self):
        c = self.connection()
        self.bir(c, root_unit=0)


class DumpAllUnitsToRiak(BirimAgaci):
    """
    Dump All Units To Riak by root unit id. If scheduler's self.request.raw_request is empty, then root_unit = 0
    This service runs every night at 03.14 am to sync yoksis data for all universities.
    """

    def handle(self):
        if self.request.raw_request:
            root_unit = self.request.raw_request
        else:
            root_unit = 0
        conn = self.connection()
        self.bir(conn, root_unit=root_unit)


class DumpUnitsToUlakbusUnitModel(BirimAgaci):
    """
     Dump All Units To Ulakbus auth.Unit Model by UID.
     """

    def handle(self):

        if self.request.raw_request:
            root_unit = self.request.raw_request
        else:
            root_unit = UID
        conn = self.connection()
        self.bir(conn, root_unit=root_unit)

    def birim_kaydet(self, birim_id):
        from ulakbus.models.auth import Unit
        data = self.birim_detaylari()
        u, is_new = Unit.objects.get_or_create(yoksis_no=birim_id)

        u.name, should_save = (u.name, False) if u.name == data['birim_adi'] else (data['birim_adi'], True)
        u.long_name, should_save = (u.long_name, False) if u.name == data['birim_uzun_adi'] else (
            data['birim_uzun_adi'], True)
        u.city_code, should_save = (u.city_code, False) if u.name == data['il_kodu'] else (data['il_kodu'], True)
        u.district_code, should_save = (u.district_code, False) if u.name == data['ilce_kodu'] else (
            data['ilce_kodu'], True)
        u.language, should_save = (u.language, False) if u.name == data['ogrenim_dili'] else (
            data['ogrenim_dili'], True)
        u.english_name, should_save = (u.english_name, False) if u.name == data['birim_adi_ingilizce'] else (
            data['birim_adi_ingilizce'], True)
        u.parent_unit_no, should_save = (u.parent_unit_no, False) if u.name == data['bagli_oldugu_birim_id'] else (
            data['bagli_oldugu_birim_id'], True)
        u.learning_duration, should_save = (u.learning_duration, False) if u.name == data['ogrenim_suresi'] else (
            data['ogrenim_suresi'], True)
        u.osym_code, should_save = (u.osym_code, False) if u.name == data['klavuz_kodu'] else (
            data['klavuz_kodu'], True)
        u.learning_type, should_save = (u.learning_type, False) if u.name == data['ogrenim_turu'] else (
            data['ogrenim_turu'], True)
        u.unit_type, should_save = (u.unit_type, False) if u.name == data['birim_turu_adi'] else (
            data['birim_turu_adi'], True)

        new_situation = False if data['aktif'] in ['Kapalı', 'Pasif'] else True
        u.is_active, should_save = (u.is_active, False) if u.is_active == new_situation else (new_situation, True)

        u.is_active = False if u.current_situation in ['Kapalı', 'Pasif'] else True

        if is_new or should_save:
            u.is_academic = True
            u.uid = UID
            u.save()



# -*- coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Senkronizasyon Servisi

Hitap senkronizasyon servislerinin kalıtılacağı
abstract HITAP Sorgula servisini içeren modül.

Bu servis şu işlemleri gerçekleştirir:

    * Hitap'taki kayıt yerelde yoksa, yerele kaydedilir.
    * Yereldeki kayıt Hitap'ta yoksa ve sync değeri 1 (senkronize) görünüyorsa,
      Hitap'ta bu kayıt silinmiş demektir ve yerelde de silinir.

``sync`` alanı şu değerlerde bulunabilir:

    * 1: Kayıt Hitap ile senkronize
    * 2: Yerel kayıt güncellendi, Hitap güncellenecek
    * 3: Yerel kayıt silindi, Hitap kaydı silinecek
    * 4: Yeni bir yerel kayıt oluşturuldu, Hitap'a gönderilecek.


Attributes:
    H_USER (str): Hitap kullanıcı adı
    H_PASS (str): Hitap kullanıcı şifresi


Example:
    Servise JSON nesnesi kullanılarak istek gönderilmesi:

    .. code-block:: json

        $ curl http://localhost:11223/hizmet-okul-sync -d '{"tckn": "tckn"}'

"""

from zato.server.service import Service
from zato.common import DATA_FORMAT
import os
import urllib2
import socket
from json import loads, dumps
from six import iteritems
from ulakbus.models.personel import Personel


H_USER = os.environ["HITAP_USER"]
H_PASS = os.environ["HITAP_PASS"]


class HITAPSync(Service):
    """
    Hitap Sync servislerinin kalıtılacağı abstract Zato servisi.

    Senkronizasyon servisleri gerekli girdileri (Hitap username, Hitap password, tckn)
    Hitap sorgulama servislerine gönderip dönecek cevaba göre
    yereldeki kayıtları güncelleyebilmektedirler.

    Attributes:
        sorgula_service (str): İlgili Hitap sorgu servisinin adı
        model (Model): Hitap'taki kaydın karşılığı olan Model

    """

    def __init__(self):
        self.sorgula_service = ''
        self.model = None
        super(HITAPSync, self).__init__()

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Servise gelen istekten kimlik numarası (tckn) bilgisini alır ve
        Hitap sorgulama servisine gidecek isteği hazırlayacak
        ve gelen cevabı elde edecek olan sync_hitap_data fonksiyonunu çağırır.

        """

        self.logger.info("zato service started to work.")
        tckn = self.request.payload['tckn']
        self.personel = Personel.objects.filter(tckn=tckn)[0]

        self.sync_hitap_data(tckn)

    def get_hitap_dict(self, tckn):
        """
        İlgili servise ait sorgulama servisini çağırarak
        gerekli Hitap verisini elde eder.

        Args:
            tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

        Returns:
            (List[dict], Bool):
                Hitap verisini yerele uygun biçimde tutan sözlük listesi ve
                Sorgu sonucunda hata olup olmadığı bilgisi

        """

        response = self.invoke(self.sorgula_service, dumps({'tckn': tckn}),
                               data_format=DATA_FORMAT.JSON, as_bunch=True)

        has_error = True if response["status"] == 'error' else False
        hitap_dict = loads(response["result"])

        return hitap_dict, has_error

    def save_hitap_data_db(self, hitap_data):
        """
        Hitap servisinden gelen kayıt yerelde yoksa, sync değeri 1 (senkronize)
        olacak şekilde kaydı veritabanına kaydeder.

        Args:
            hitap_data (dict): Hitap verisini yerele uygun biçimde tutan sözlük

        """

        obj = self.model()
        for hk, hv in iteritems(hitap_data):
            setattr(obj, hk, hv)

        obj.sync = 1
        obj.personel = self.personel
        obj.save()
        self.logger.info("hitaptaki kayit yerele kaydedildi. kayit no => "
                         + str(obj.kayit_no))

    def delete_hitap_data_db(self, kayit_no):
        """
        Yereldeki kayıt Hitap'ta yoksa ve sync değeri 1 (senkronize) görünüyorsa,
        Hitap'ta bu kayıt silinmiş demektir ve yerelde de silinir.

        Args:
            kayit_no (str): Hitap kaydının kayıt numarası

        """

        obj = self.model.objects.get(kayit_no=kayit_no)

        if obj.sync == 1:
            obj.delete()
            self.logger.info("yereldeki sync kayit hitapta yok, kayit silindi."
                             "kayit no => " + str(obj.kayit_no))

    def sync_hitap_data(self, tckn):
        """
        Yereldeki kayıtları Hitap servisinden gelen kayıtlara göre senkronize eder.
        tckn bilgisine göre Hitap kayıtları ve yereldeki kayıtları getirilir ve:

            * Hitap'taki kayıt yerelde yoksa, yerele kaydedilir.
            * Yereldeki kayıt Hitap'ta yoksa ve sync değeri 1 (senkronize) görünüyorsa,
              Hitap'ta bu kayıt silinmiş demektir ve yerelde de silinir.

        Args:
            tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

        Raises:
            AttributeError: İlgili servis veya bean Hitap'ta bulunmayabilir.
            urllib2.URLError: Servis yanıt vermiyor.
            socket.error: Riak bağlantı hatası.
            Exception: Beklenmeyen hata.

        """

        # get hitap data
        hitap_dict, has_error = self.get_hitap_dict(tckn)

        if has_error:
            self.logger.info("Hitap kaydi sorgulama hatasi.")
            status = "error"
            result = "Service unavailable!"
        else:
            try:
                # get kayit no list from db
                kayit_no_list = [record.kayit_no for record in
                                 self.model.objects.filter(tckn=tckn)]
                self.logger.info("yereldeki kayit sayisi: " + str(len(kayit_no_list)))
                self.logger.info("hitaptan gelen kayit sayisi: " + str(len(hitap_dict)))

                # compare hitap data with db
                for hitap_record in hitap_dict:
                    hitap_kayit_no = hitap_record['kayit_no']

                    # if hitap data is not in db, create an object and save to db.
                    if hitap_kayit_no not in kayit_no_list:
                        self.save_hitap_data_db(hitap_record)
                    # if in db, don't touch.
                    else:
                        kayit_no_list.remove(hitap_kayit_no)

                # if there are still some in sync records which are not in hitap, delete them.
                for model_kayit_no in kayit_no_list:
                    self.delete_hitap_data_db(model_kayit_no)

                status = "ok"
                result = "Synchronisation completed successfully."

            except AttributeError:
                self.logger.exception("AttributeError")
                status = "error"
                result = "AttributeError"
            except socket.error:
                self.logger.exception("Riak connection refused!")
                status = "error"
                result = "Riak connection refused!"
            except urllib2.URLError:
                self.logger.exception("Service unavailable!")
                status = "error"
                result = "Service unavailable!"
            except Exception:
                self.logger.exception("Unexpected error!")
                status = "error"
                result = "Unexpected error!"

        self.logger.info(result)
        self.response.payload = {'status': status, 'result': result}



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Askerlik Sorgula

Hitap üzerinden personelin askerlik bilgilerinin sorgulamasını yapar.

"""

from ulakbus.services.personel.hitap.hitap_sorgula import HITAPSorgula


class HizmetAskerlikGetir(HITAPSorgula):
    """
    HITAP Sorgulama servisinden kalıtılmış Askerlik Bilgisi Sorgulama servisi

    """

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            bean_name (str): Hitap'tan gelen bean nesnesinin adı
            service_dict (dict): Hitap servisinden gelen kayıtların alanları,
                    ``AskerlikKayitlari`` modelinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları listede tutulmaktadır.

        """

        self.service_name = 'HizmetAskerlikSorgula'
        self.bean_name = 'HizmetAskerlikServisBean'
        self.service_dict = {
            'fields': {
                'tckn': 'tckn',
                'kayit_no': 'kayitNo',
                'askerlik_nevi': 'askerlikNevi',
                'baslama_tarihi': 'baslamaTarihi',
                'bitis_tarihi': 'bitisTarihi',
                'sayilmayan_gun_sayisi': 'sayilmayanGunSayisi',
                'subay_okulu_giris_tarihi': 'subayOkuluGirisTarihi',
                'astegmen_nasp_tarihi': 'astegmenNaspTarihi',
                'tegmen_nasp_tarihi': 'tegmenNaspTarihi',
                'sinif_okulu_sicil': 'sinifOkuluSicil',
                'muafiyet_neden': 'muafiyetNeden',
                'gorev_yeri': 'gorevYeri',
                'subayliktan_erlige_gecis_tarihi': 'subayliktanErligeGecisTarihi',
                'kita_baslama_tarihi': 'kitaBaslamaTarihi',
                'kita_bitis_tarihi': 'kitaBitisTarihi',
                'kurum_onay_tarihi': 'kurumOnayTarihi'
            },
            'date_filter': ['baslama_tarihi', 'bitis_tarihi', 'subay_okulu_giris_tarihi',
                            'astegmen_nasp_tarihi', 'tegmen_nasp_tarihi',
                            'subayliktan_erlige_gecis_tarihi', 'kita_baslama_tarihi',
                            'kita_bitis_tarihi', 'kurum_onay_tarihi'],
            'required_fields': ['tckn']
        }
        super(HizmetAskerlikGetir, self).handle()



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
from zengine.forms import JsonForm

from ulakbus.models import OgrenciProgram, Ogrenci, Role, User, AbstractRole
from zengine.forms import fields
from zengine.notifications import Notify
from zengine.views.crud import CrudView

ABSTRACT_ROLE_LIST = [
    "Lisans Programı Öğrencisi - Aktif",
    "Lisans Programı Öğrencisi - Kayıt Dondurmuş",

    "Ön Lisans Programı Öğrencisi - Aktif",
    "Ön Lisans Programı Öğrencisi - Kayıt Dondurmuş",

    "Yüksek Lisans Programı Öğrencisi - Aktif",
    "Yüksek Lisans Programı Öğrencisi - Kayıt Dondurmuş",

    "Doktora Programı Öğrencisi - Aktif",
    "Doktora Programı Öğrencisi - Kayıt Dondurmuş",

]

ABSTRACT_ROLE_LIST_SILINMIS = [
    "Lisans Programı Öğrencisi - Kayıt Silinmiş",
    "Ön Lisans Programı Öğrencisi - Kayıt Silinmiş",
    "Yüksek Lisans Programı Öğrencisi - Kayıt Silinmiş",
    "Doktora Programı Öğrencisi - Kayıt Silinmiş",
]


class KayitSil(CrudView):
    """ Kayıt Silme İş Akışı

    Kayıt silme iş akışı 8 adımdan oluşmaktadır.
    * Kaydı Kontrol Et
    * Kaydı Silinen Öğrenci
    * Kayıt Silme İşlemini Onayla
    * Kayıt Silme İşleminden Vazgeç
    * Fakülte Karar No
    * Ayrılma nedenini seç
    * Öğrenci programı seç
    * Bilgi ver

    Kayıt silme iş akışında öğrencinin kayıtlı olduğu öğrenci programları silinmez,
    öğrencinin kayıtlı olduğu öğrenci programlarının ayrılma nedeni ve öğrencilik
    statüsü field'larına değerler atanır.

    Bu iş akışında kullanılan metotlar şu şekildedir.

    Kaydı Kontrol Et:
    Öğrencinin kaydının silinip silinmediğini kontrol eder.

    Kaydı Silinen Öğrenci:
    Öğrencinin kaydı silinmişse kaydın silindiğine dair bilgi mesajı ekrana basılır.

    Kayıt Silme İşlemini Onayla:
    Personel kayıt silme işlemine devam etmek isteyip istemediği sorulur.

    Kayıt Silme İşleminden Vazgeç:
    Personelin kayıt silme işleminden vazgeçmesi durumunda ekrana silme
    işlemin iptal edildiğine dair bilgi mesajı basılır.

    Fakülte Karar No:
    Fakülte Yönetim Kurulu tarafından belirlenen karar no girilir.

    Ayrılma nedeni seç:
    Öğrencinin ayrılma nedeni seçilir.

    Öğrenci programı seç:
    Öğrencinin kayıtlı olduğu öğrenci programlarının ayrılık nedeni ve öğrencilik statüsü
    field'larına değerler atanır.

    Bilgi ver:
    Danışmana ve öğrenciye kayıt silme işlemi ile ilgili bilgi verilir.
    Kayıt silme iş akışının son adımıdır. Bu adımdan sonra iş akışı sona erer.

    Bu sınıf ``CrudView`` extend edilerek hazırlanmıştır. Temel model ``OgrenciProgram``
    modelidir. Meta.model bu amaçla kullanılmıştır.

    Adımlar arası geçiş manuel yürütülmektedir.

    """

    class Meta:
        model = 'OgrenciProgram'

    def kontrol(self):
        """
        Öğrencinin kaydının silinip silinmediğini kontrol eder.

        """

        self.current.task_data['command'] = 'kaydi_silinen_ogrenci'
        self.current.task_data['ogrenci_id'] = self.current.input['id']
        ogrenci = Ogrenci.objects.get(self.current.task_data['ogrenci_id'])
        programlar = OgrenciProgram.objects.filter(ogrenci=ogrenci)
        self.current.task_data['roles'] = []
        for program in programlar:
            roles = Role.objects.filter(user=ogrenci.user, unit=program.program.birim)
            for role in roles:
                self.current.task_data['roles'].append(role.abstract_role.name)
                name = role.abstract_role.name
                if name not in ABSTRACT_ROLE_LIST_SILINMIS and name in ABSTRACT_ROLE_LIST:
                    self.current.task_data['command'] = 'kayit_silme_islemini_onayla'
                    break

    def kaydi_silinen_ogrenci(self):
        """
        Öğrencinin kaydı silinmiş ise öğrenci kaydının silindiğine dair bilgi
        mesajı ekrana basılır.

        """

        ogrenci = Ogrenci.objects.get(self.current.task_data['ogrenci_id'])
        self.current.output['msgbox'] = {
            'type': 'warning', "title": 'Kayıt Silme Başarılı',
            "msg": ' %s adlı öğrencinin kaydı daha önceden silinmiştir.' % ogrenci

        }

    def kayit_silme_islemini_onayla(self):
        """
        Personele kayıt silme işlemine devam etmek isteyip istemediği sorulur.

        """

        ogrenci = Ogrenci.objects.get(self.current.task_data['ogrenci_id'])
        _form = JsonForm(current=self.current,
                         title='Kayıt Silme İşlemini Onaylayınız.')
        _form.help_text = '%s adlı öğrencinin %s rollerini silmek üzerisiniz. Emin misiniz?' % (
        ogrenci, '-'.join(
            name for name in self.current.task_data['roles']))
        _form.kaydet = fields.Button('Onayla', flow='fakulte_yonetim_karari')
        _form.vazgecme = fields.Button('Vazgeç', flow='kayit_silme_isleminden_vazgec')
        self.form_out(_form)

    def kayit_silme_isleminden_vazgec(self):
        """
        Personelin kayıt silme işleminden vazgeçmesi durumunda ekrana silme işleminin
        iptal edildiğine dair bilgi mesajı basılır.

        """

        self.current.output['msgbox'] = {
            'type': 'warning', "title": 'Kayıt Silme İşlemi',
            "msg": 'Kayıt silme işlemi iptal edilmiştir.'

        }

    def fakulte_yonetim_karari(self):
        """
        Fakülte Yönetim Kurulu tarafından belirlenen karar no girilir.

        """

        # TODO: Fakülte yönetim kurulunun kararı loglanacak.
        _form = JsonForm(current=self.current,
                         title='Fakülte Yönetim Kurulunun Karar Numarasını Giriniz.')
        _form.karar = fields.String('Karar No', index=True)
        _form.kaydet = fields.Button('Kaydet')
        self.form_out(_form)

    def ayrilma_nedeni_sec(self):
        """
        Ayrılma nedenlerini form içinde listelenir. Listelenen ayrılma nedenlerinden biri
        kullanıcı tarafından seçilir.

        """

        _form = JsonForm(current=self.current, title='Öğrencinin Ayrılma Nedenini Seçiniz')
        _form.ayrilma_nedeni = fields.Integer(choices=self.object.get_choices_for('ayrilma_nedeni'))
        _form.aciklama = fields.Text("Açıklama Yazınız", required=True)
        _form.sec = fields.Button("Seç")
        self.form_out(_form)

    def ogrenci_program_sec(self):
        """
        Öğrencinin kayıtlı olduğu öğrenci programların ayrılma nedeni field'larına, ayrılma
        nedeni seç adımındaki ayrılma nedeni atanır.

        Öğrencinin kayıtlı olduğu öğrenci programların öğrencilik statüsüne, ``Kaydı silinmiştir``
        statüsü eklenmiştir.

        Öğrencinin rolü kayıtlı olduğu birimin tipine (program, lisans programı, doktora programı )
        göre değiştirilir.

        Eğer öğrencinin okulda başka bir rolü (kütüphane çalışanı,spor salonu çalışanı) var ise
        admine bilgi mesajı yollanır.

        """

        ogrenci = Ogrenci.objects.get(self.current.task_data['ogrenci_id'])
        programlar = OgrenciProgram.objects.filter(ogrenci_id=self.current.task_data['ogrenci_id'])
        for program in programlar:
            program.ayrilma_nedeni = self.current.input['form']['ayrilma_nedeni']
            # todo: elle vermek yerine daha iyi bir yol dusunelim
            program.ogrencilik_statusu = 21
            program.save()
            roles = Role.objects.filter(user=ogrenci.user, unit=program.program.birim)
            for role in roles:
                if role.abstract_role.name in ABSTRACT_ROLE_LIST:
                    if role.unit.unit_type == 'Program':
                        abstract_role = AbstractRole.objects.get(
                            name=ABSTRACT_ROLE_LIST_SILINMIS[0])
                        role.abstract_role = abstract_role
                        role.save()
                    elif role.unit.unit_type == 'Yüksek Lisans Programı':
                        abstract_role = AbstractRole.objects.get(
                            name=ABSTRACT_ROLE_LIST_SILINMIS[2])
                        role.abstract_role = abstract_role
                        role.save()
                    elif role.unit.unit_type == 'Doktora Programı':
                        abstract_role = AbstractRole.objects.get(
                            name=ABSTRACT_ROLE_LIST_SILINMIS[3])
                        role.abstract_role = abstract_role
                        role.save()
                    else:
                        abstract_role = AbstractRole.objects.get(
                            name=ABSTRACT_ROLE_LIST_SILINMIS[1])
                        role.abstract_role = abstract_role
                        role.save()

        ogrenci_rolleri = Role.objects.filter(user=ogrenci.user)
        for role in ogrenci_rolleri:
            if role.abstract_role.name not in ABSTRACT_ROLE_LIST_SILINMIS:
                title = 'Kayıt Silme'
                msg = """%s adlı öğrencinin kaydı silinmiştir.
                         Öğrenci farklı rollere sahiptir.""" % ogrenci

                # TODO: sistem yoneticisine bilgi ver.
                usr = User.objects.get(username='ulakbus')
                self.notify(usr, msg=msg, title=title)
                break

    def bilgi_ver(self):
        """
        Kayıt silme iş akışı tamamlandıktan sonra danışmana ve öğrenciye bilgi verilir.
        Kayıt silme işleminin tamamlandığına dair ekrana çıktı verir.

        """
        ogrenci = Ogrenci.objects.get(self.current.task_data['ogrenci_id'])
        ogrenci_program = OgrenciProgram.objects.filter(ogrenci=ogrenci)
        self.current.output['msgbox'] = {
            'type': 'warning', "title": 'Kayıt Silme',
            "msg": 'Öğrencinin kaydı %s nedeniyle silinmiştir.' % self.current.input['form'][
                'aciklama']
        }
        title = 'Kayıt Silme'
        msg = '%s adlı öğrencinin kaydı %s nedeniyle silinmiştir.' % (
            ogrenci, self.current.input['form']['aciklama'])

        for program in ogrenci_program:
            self.notify(program.danisman.user, title=title, msg=msg)

        self.notify(ogrenci.user, title=title, msg=msg)

    @staticmethod
    def notify(user, msg, title):
        if user:
            Notify(user.key).set_message(msg=msg, title=title, typ=Notify.TaskInfo)




# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Mahkeme Sorgula

Hitap üzerinden personelin mahkeme bilgilerinin sorgulamasını yapar.

"""

from ulakbus.services.personel.hitap.hitap_sorgula import HITAPSorgula


class HizmetMahkemeGetir(HITAPSorgula):
    """
    HITAP Sorgulama servisinden kalıtılmış Mahkeme Bilgisi Sorgulama servisi

    """

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            bean_name (str): Hitap'tan gelen bean nesnesinin adı
            service_dict (dict): Hitap servisinden gelen kayıtların alanları,
                    ``HizmetMahkeme`` modelinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları ve servis tarafında gerekli olan
                    alanlar listede tutulmaktadır.

        """

        self.service_name = 'HizmetMahkemeSorgula'
        self.bean_name = 'HizmetMahkemeServisBean'
        self.service_dict = {
            'fields': {
                'tckn': 'tckn',
                'kayit_no': 'kayitNo',
                'mahkeme_ad': 'mahkemeAd',
                'sebep': 'sebep',
                'karar_tarihi': 'kararTarihi',
                'karar_sayisi': 'kararSayisi',
                'kesinlesme_tarihi': 'kesinlesmeTarihi',
                'asil_dogum_tarihi': 'asilDogumTarihi',
                'tashih_dogum_tarihi': 'tashihDogumTarihi',
                'asil_ad': 'asilAd',
                'tashih_ad': 'tashihAd',
                'asil_soyad': 'asilSoyad',
                'tashih_soyad': 'tashihSoyad',
                'gecerli_dogum_tarihi': 'gecerliDogumTarihi',
                'aciklama': 'aciklama',
                'gun_sayisi': 'gunSayisi',
                'kurum_onay_tarihi': 'kurumOnayTarihi'
            },
            'date_filter': ['karar_tarihi', 'kesinlesme_tarihi', 'asil_dogum_tarihi',
                            'tashih_dogum_tarihi', 'gecerli_dogum_tarihi', 'kurum_onay_tarihi'],
            'required_fields': ['tckn']
        }
        super(HizmetMahkemeGetir, self).handle()



#!/usr/bin/env python
# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.


from zengine.management_commands import *
from ulakbus.lib.unitime import *
from ulakbus.lib.fake import *

class CreateUser(Command):
    CMD_NAME = 'create_user'
    HELP = 'Creates a new user'
    PARAMS = [
        {'name': 'username', 'required': True, 'help': 'Login username'},
        {'name': 'password', 'required': True, 'help': 'Login password'},
        {'name': 'abstract_role', 'default': 'BaseAbsRole', 'help': 'Name of the AbstractRole'},
        {'name': 'super', 'action': 'store_true', 'help': 'This is a super user'},
        {'name': 'permission_query', 'default': "code:crud* OR code:login* OR code:logout*",
         'help': 'Permissions which will be returned from this query will be granted to the user. '
                 'Defaults to: "code:crud* OR code:login* OR code:logout*"'},
    ]

    def run(self):
        from ulakbus.models import AbstractRole, User, Role, Permission
        if User.objects.filter(username=self.manager.args.username).count():
            print("User already exists!")
            return
        abs_role, new = AbstractRole.objects.get_or_create(name=self.manager.args.abstract_role)
        user = User(username=self.manager.args.username, superuser=self.manager.args.super)
        user.set_password(self.manager.args.password)
        user.save()
        role = Role(user=user, abstract_role=abs_role)
        role.save()
        perm_list = []
        for perm in Permission.objects.raw(self.manager.args.permission_query):
            role.Permissions(permission=perm)
            perm_list.append(perm.name)
        role.save()
        user_type = 'super user' if self.manager.args.super else 'user'
        print("New %s created with these permissions: \n\n%s" % (user_type, "\n".join(perm_list)))


class LoadFixture(Command):
    CMD_NAME = 'load_fixture'
    HELP = 'Load fixtures from given json file or files in given directory and ' \
           'dumps into ulakbus_settings_fixtures bucket, overwriting data of all existing keys.'
    PARAMS = [
        {'name': 'path', 'required': True, 'help': 'Load fixture file or directory'},
    ]

    def run(self):
        from pyoko.db.connection import client
        import os

        fixture_bucket = client.bucket_type('catalog').bucket('ulakbus_settings_fixtures')
        path = self.manager.args.path

        if os.path.isdir(path):
            from glob import glob
            for fixture_file in glob(os.path.join(path, "*.json")):
                self.dump(fixture_file, fixture_bucket)
        else:
            self.dump(path, fixture_bucket)

    @staticmethod
    def dump(fixture_file, fixture_bucket):
        try:
            with open(fixture_file) as f:
                import json
                try:
                    fixtures = json.load(f)
                    for fix in fixtures:
                        f = fixture_bucket.get(fix)
                        f.data = fixtures[fix]
                        print("%s: %s stored.." % (fix, fixtures[fix]))
                        f.store()
                except ValueError as e:
                    print("please validate your json file: %s" % e)
        except IOError:
            print("file not found: %s" % fixture_file)


class GenerateBuildingList(Command):
    CMD_NAME = 'generate_buildings'
    HELP = 'Generates fake Building model objects from Unit Faculties'
    PARAMS = []

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        fake = FakeDataGenerator()
        bina_list, room_list = fake.yeni_bina()
        print("%s adet bina oluşturuldu, oluşturulan binalar: %s\n" % (len(bina_list), bina_list))
        print("%s adet oda oluşturuldu, oluşturulan odalar: %s\n" % (len(room_list), room_list))

class GenerateRandomPersonel(Command):
    CMD_NAME = 'random_personel'
    HELP = 'Generates Random Personel'
    PARAMS = [

        {'name': 'length', 'required': False, 'help': 'Amount of random personel', 'default': 1},

    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        length = int(self.manager.args.length)
        fake = FakeDataGenerator()
        fake.yeni_personel(personel_say=length)


class GenerateRandomOkutman(Command):
    CMD_NAME = 'random_okutman'
    HELP = 'Generates Random Okutman From Personel Objects'
    PARAMS = [
        {'name': 'personel', 'required': True, 'help': 'Personel object'},
    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        personel = self.manager.args.personel
        fake = FakeDataGenerator()
        fake.yeni_okutman(personel=personel)


class GenerateRandomHariciOkutman(Command):
    CMD_NAME = 'random_harici_okutman'
    HELP = 'Generates Random Okutman From Personel Objects'
    PARAMS = [

        {'name': 'length', 'required': True, 'help': 'Amount of random harici okutman', 'default': 1},

    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        length = int(self.manager.args.length)
        fake = FakeDataGenerator()
        fake.yeni_harici_okutman(harici_okutman_say=length)


class GenerateRandomOgrenci(Command):
    CMD_NAME = 'random_ogrenci'
    HELP = 'Generates Random Ogrenci Model Objects'
    PARAMS = [
        {'name': 'length', 'required': False, 'help': 'Amount of random ogrenci', 'default': 1},
    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        length = int(self.manager.args.length)
        fake = FakeDataGenerator()
        ogrenci_list = fake.yeni_ogrenci(ogrenci_say=length)
        print("Toplam %s adet ogrenci oluşturuldu, oluşturulan öğrenci listesi : %s" % (length, ogrenci_list))


class GenerateProgramList(Command):
    CMD_NAME = 'generate_programs'
    HELP = 'Generates Program Records From Unit Object'
    PARAMS = [
        {'name': 'length', 'required': False, 'help': 'Amount of random program', 'default': 1},
        {'name': 'yoksis_program', 'required': True, 'help': 'Unit object'},
    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        length = int(self.manager.args.length)
        yoksis_program = self.manager.args.yoksis_program
        fake = FakeDataGenerator()
        fake.yeni_program(yoksis_program=yoksis_program, program_say=length)


class GenerateDersList(Command):
    CMD_NAME = 'generate_ders'
    HELP = 'Generates fake Ders model objects'
    PARAMS = [
        {'name': 'length', 'required': False, 'help': 'Amount of random program', 'default': 1},
        {'name': 'program', 'required': True, 'help': 'Program object'},
        {'name': 'personel', 'required': True, 'help': 'Personel object'},
    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        length = int(self.manager.args.length)
        personel = self.manager.args.personel
        program = self.manager.args.program
        fake = FakeDataGenerator()
        fake.yeni_ders(program=program, personel=personel, ders_say=length)

environ['PYOKO_SETTINGS'] = 'ulakbus.settings'
environ['ZENGINE_SETTINGS'] = 'ulakbus.settings'

if __name__ == '__main__':
    ManagementCommands()



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
"""Ders Modülü

İlgili İş Akışlarına ait sınıf ve metotları içeren modüldür.

Ders Ekle ve Ders Şubelendirme iş akışlarının yürütülmesini sağlar.

"""

from collections import OrderedDict

import time

from pyoko import ListNode
from pyoko.db.adapter.db_riak import BlockSave, BlockDelete
from ulakbus.models.ogrenci import DegerlendirmeNot, OgrenciProgram
from ulakbus.models.ogrenci import Program, Okutman, Ders, Sube, Sinav, OgrenciDersi, Donem
from zengine import forms
from zengine.forms import fields
from zengine.views.crud import CrudView
from ulakbus.lib.view_helpers import prepare_choices_for_model


def okutman_choices():
    """Okutman Seçenekleri

    Returns:
        prepare_choices_for_model methodundan dönen değerleri dictionary
        formatında

    """

    return [{'name': name, 'value': value} for value, name in prepare_choices_for_model(Okutman)]


class ProgramBilgisiForm(forms.JsonForm):
    """
    ``DersEkle`` sınıfı için form olarak kullanılacaktır. Form,
    include listesinde, aşağıda tanımlı alana sahiptir.

    """

    class Meta:
        include = ['program']

    sec = fields.Button("Seç", cmd="program_sec")


class DersBilgileriForm(forms.JsonForm):
    """
    ``DersEkle`` sınıfı için form olarak kullanılacaktır. Form,
    include listesinde, aşağıda tanımlı alanlara sahiptir.

    """

    class Meta:
        include = ['ad', 'kod', 'tanim', 'aciklama', 'onkosul', 'uygulama_saati', 'teori_saati',
                   'ects_kredisi',
                   'yerel_kredisi', 'zorunlu', 'ders_dili', 'ders_turu', 'ders_amaci',
                   'ogrenme_ciktilari',
                   'ders_icerigi', 'ders_kategorisi', 'ders_kaynaklari', 'ders_mufredati',
                   'verilis_bicimi', 'donem',
                   'ders_koordinatoru']

    kaydet = fields.Button("Kaydet", cmd="kaydet")


class DersDegerlendirmeForm(forms.JsonForm):
    class Meta:
        include = ['Degerlendirme']

    kaydet = fields.Button("Kaydet", cmd="degerlendirme_kaydet", flow="end")
    kaydet_yeni_kayit = fields.Button("Kaydet/Yeni Kayıt Ekle", cmd="kaydet", flow="start")


class DersEkle(CrudView):
    """Ders Ekle İş Akışı

    Ders Ekle, aşağıda tanımlı iş akışı adımlarını yürütür.

    - Program Formunu Listele
    - Program Seç
    - Ders Bilgilerini Gir
    - Kaydet
    - Kaydet/Yeni Kayıt Ekle

     Bu iş akışında kullanılan metotlar şu şekildedir:

     Program Formunu Listele:
        CrudView list metodu kullanılmıştır. Bu metot default olarak
        tanımlanmıştır. Program Bilgisi formunu listeler.

     Seç:
        Kayıtlı programlardan birini seçer.

     Ders Bilgilerini Gir:
        Ders Bilgileri formundaki alanlar doldurulur.

     Kaydet/Yeni Kayıt Ekle:
        Seçilen programı için  girilen ders bilgilerini kaydeder.
        Bu adımdan sonra iş akışı sona erer.

     Bu sınıf ``CrudView`` extend edilerek hazırlanmıştır. Temel model
     ``Ders`` modelidir. Meta.model bu amaçla kullanılmıştır.

     Adımlar arası geçiş manuel yürütülmektedir.

    """

    class Meta:
        model = "Ders"

    def program_sec(self):
        """Program Seçimi"""

        self.set_form_data_to_object()
        self.current.task_data['program_id'] = self.object.program.key

    def degerlendirme_form(self):
        ders = Ders.objects.get(self.current.task_data['ders_id'])
        self.form_out(DersDegerlendirmeForm(ders, current=self.current))

    def kaydet(self):
        self.set_form_data_to_object()
        self.object.program = Program.objects.get(self.current.task_data['program_id'])
        self.save_object()
        self.current.task_data['ders_id'] = self.object.key
        # self.current.task_data['next'] = self.current.input['next']

    def ders_bilgileri(self):
        """Ders Bilgileri Formu"""

        self.form_out(DersBilgileriForm(self.object, current=self.current))

    def program_bilgisi(self):
        """Program Bilgisi Formu"""

        self.form_out(ProgramBilgisiForm(self.object, current=self.current))


class SecimForm(forms.JsonForm):
    """

    ``DersSubelendirme`` sınıfı için form olarak kullanılacaktır.

    """

    sec = fields.Button("Sec", cmd="ders_sec")


class ProgramForm(forms.JsonForm):
    """
    ``DersSubelendirme`` sınıfı için form olarak kullanılacaktır.

    """

    sec = fields.Button("Sec", cmd="ders_sec")


class SubelendirmeForm(forms.JsonForm):
    """
    ``DersSubelendirme`` sınıfı için form olarak kullanılacaktır.
    """

    kaydet_ders = fields.Button("Kaydet ve Ders Seçim Ekranına Dön", cmd="subelendirme_kaydet",
                                flow="ders_okutman_formu")
    program_sec = fields.Button("Kaydet ve Program Seçim Ekranına Dön", cmd="subelendirme_kaydet",
                                flow="program_sec")
    bilgi_ver = fields.Button("Tamamla ve Hocaları Bilgilendir", cmd="subelendirme_kaydet",
                              flow="bilgi_ver")

    class Subeler(ListNode):
        ad = fields.String('Sube Adi')
        kontenjan = fields.Integer('Sube Kontenjani')
        dis_kontenjan = fields.Integer('Sube Dis Kontenjani')
        okutman = fields.String('Okutman', choices=okutman_choices)


class NotGirisForm(forms.JsonForm):
    class Meta:
        inline_edit = ['degerlendirme', 'aciklama']

    class Ogrenciler(ListNode):
        ogrenci_no = fields.String('No')
        ad_soyad = fields.String('Ad Soyad')
        degerlendirme = fields.Integer('Not')
        aciklama = fields.String('Aciklama')
        key = fields.String('KKKKey', hidden=True)


class DersSubelendirme(CrudView):
    """Ders Şubelendirme İş Akışı

     Ders Şubelendirme, aşağıda tanımlı iş akışı adımlarını yürütür.

     - Program Formunu Listele
     - Program Seç
     - Ders Şubelendir
     - Şube Formunu Listele
     - Kaydet ve Ders Seçim Ekranına Dön
     - Kaydet ve Program Seçim Ekranına Dön
     - Tamamla ve Hocaları Bilgilendir

    Bu iş akışında kullanılan metotlar şu şekildedir:

    Program  Formunu Listele:
        CrudView list metodu kullanılmıştır.Bu metot default olarak
        tanımlanmıştır. Program formunu listeler.

    Seç:
        Kayıtlı programlardan birini seçer. Seçilen program kayıtlı
        dersleri döndürür.

    Şubelendir:
        Şubelendirelecek ders seçilir.

    Şube Formunu Listele:
        Şubelenmiş dersin şube detaylarını gösterir, seçilen dersi
        şubelendirir.

    Kaydet ve Ders Seçim Ekranına Dön:
        Şubelendirilmiş dersi kaydeder ve ders seçim ekranından geri
        döner. İş akışına ders seçim ekranından devam eder.

    Kaydet ve Program Seçim Ekranına Dön:
        Şubelendirilmiş dersi kaydeder ve program seçim  ekranına geri
        döner. İş akışı Program formundan devam eder.

    Tamamla ve Hocaları Bilgilendir:
         Şubelendirilmiş dersin hocalarına, şube ve ders bilgileri
         aktarılır. Bu adımdan sonra iş akışı sona eriyor.

    Bu sınıf ``CrudView`` extend edilerek hazırlanmıştır. Temel model
    ``Sube`` modelidir. Meta.model bu amaçla kullanılmıştır.

    Adımlar arası geçiş manuel yürütülmektedir.

    """

    class Meta:
        model = "Sube"

    def program_sec(self):
        """Program Seçim Adımı

        Programlar veritabanından çekilip, açılır menu içine
        doldurulur.

        """

        _form = ProgramForm(current=self.current)
        choices = prepare_choices_for_model(Program)
        _form.program = fields.Integer(choices=choices)
        self.form_out(_form)

    def ders_sec(self):
        """Ders Seç Adımı

        Seçilen programa bağlı dersler, eğer daha önce şubelendirme
        yapıldıysa şube listesiyle birlikte ekrana getirilir.

        Formun herbir satırı ders ve şubeleri ile birlikte, ilgili
        ders için şubelendirme formuna bağlı bir butondan oluşur.

        Şubelendirme butonunun ``object_key`` i ``sube`` olarak
        tanımlanmıştır. Sonraki adımda bu key yardımıyla şube formu,
        ekrana getirilir. ``ders_okutman_formu`` metodu içinde şu
        şekilde kullanılmıştır::

            ders = Ders.objects.get(key=self.current.input['sube'])

        """

        self.set_client_cmd('form')
        self.output['objects'] = [['Dersler'], ]

        if 'program' in self.current.input['form']:
            self.current.task_data['program'] = self.current.input['form']['program']

        p = Program.objects.get(key=self.current.task_data['program'])
        dersler = Ders.objects.filter(program=p)
        # dersler = Ders.objects.filter()
        just_created = self.current.task_data.get('just_created', [])
        just_deleted = self.current.task_data.get('just_deleted', [])
        for d in dersler:
            ders = "%s - %s (%s ECTS)" % (d.kod, d.ad, d.ects_kredisi)
            ders_subeleri = Sube.objects.filter(ders=d)
            subeler = []

            def sube_append(sube):
                if sube.key not in just_deleted:
                    subeler.append(
                        {
                            "sube_ad": sube.ad,
                            "okutman_ad": sube.okutman.ad,
                            "okutman_soyad": sube.okutman.soyad,
                            "okutman_unvan": sube.okutman.okutman.get_unvan_display(),
                            "kontenjan": sube.kontenjan,
                        }
                    )

            for sube in ders_subeleri:
                sube_append(sube)
            for ders_key, sube_key in just_created:
                if ders_key == d.key:
                    sube_append(Sube.objects.get(sube_key))

            ders_subeleri = ["""* **%s %s %s**
                                Sube: %s - Kontenjan: %s""" % (
            sb['okutman_unvan'], sb['okutman_ad'],
            sb['okutman_soyad'], sb['sube_ad'],
            sb['kontenjan']) for sb in subeler]
            item = {
                "fields": ["%s\n%s" % (ders, "\n".join(ders_subeleri)), ],
                "actions": [
                    {'name': 'Subelendir', 'cmd': 'ders_okutman_formu', 'show_as': 'button',
                     'object_key': 'sube'},
                ],
                "key": d.key
            }
            self.output['objects'].append(item)

    def ders_okutman_formu(self):
        """Ders Okutman Formu

        Şubelendirme için seçilen dersin, şube bilgileri formunu oluşturur.

        """

        # subelendirme icin secilen dersi getir
        ders = Ders.objects.get(key=self.current.input['sube'])
        # sonraki adimlar icin task data icine koy
        self.current.task_data['ders_key'] = ders.key

        # formu olusturmaya basla
        subelendirme_form = SubelendirmeForm(current=self.current,
                                             title='%s / %s dersi icin subelendirme' % (
                                                 ders.donem, ders))
        # formun sube listesini olustur
        subeler = Sube.objects.filter(ders=ders)
        for sube in subeler:
            subelendirme_form.Subeler(ad=sube.ad, kontenjan=sube.kontenjan,
                                      dis_kontenjan=sube.dis_kontenjan,
                                      okutman=sube.okutman.key)

        self.form_out(subelendirme_form)

    def subelendirme_kaydet(self):
        """Şubelendirme Kaydet

        Şubelendirme formundan gelen dataları kaydeder.

        """

        sb = self.input['form']['Subeler']
        ders = self.current.task_data['ders_key']
        mevcut_subeler = Sube.objects.filter(ders_id=ders)
        # self.current.task_data['just_created'] = []
        with BlockSave(Sube):
            for s in sb:
                okutman = s['okutman']
                sube, is_new = Sube.objects.get_or_create(okutman_id=okutman, ders_id=ders)
                # mevcut_subelerden cikar
                mevcut_subeler = list(set(mevcut_subeler) - {sube})
                sube.kontenjan = s['kontenjan']
                sube.dis_kontenjan = s['dis_kontenjan']
                sube.ad = s['ad']
                sube.donem = Donem.guncel_donem()
                sube.save()
            # if is_new:
            #     self.current.task_data['just_created'].append((ders, sube.key))
        # mevcut subelerde kalanlari sil
        with BlockDelete(Sube):
            for s in mevcut_subeler:
                s.delete()

    def bilgi_ver(self):
        """Bilgi ver

        Şubelendirme bilgileri ilgili okutmanlara iletilir.

        """

        from zengine.notifications import Notify

        just_created = self.current.task_data.get('just_created', [])
        just_deleted = self.current.task_data.get('just_deleted', [])
        ders_key = self.current.task_data['ders_key']

        title = "Şubelendirme"
        bolum_baskani = "%s %s" % (self.current.user.name, self.current.user.surname)
        msg = "Bölum Başkanı %s tarafından şubelerinizde degisiklikler yapilmistir." % bolum_baskani
        okutmanlar = []

        def notify(okutman):
            Notify(okutman.okutman.user.key).set_message(title=title, msg=msg, typ=Notify.TaskInfo)
            okutmanlar.append(okutman.__unicode__())

        for ders, sube_key in just_created:
            if ders == ders_key:
                notify(Sube.objects.get(sube_key).okutman)

        sbs = Sube.objects.filter(ders_id=self.current.task_data['ders_key'])
        for s in sbs:
            if s.key not in just_deleted:
                notify(s.okutman)

        self.current.output['msgbox'] = {
            'type': 'info', "title": 'Mesaj Iletildi',
            "msg": 'Şubelendirme Bilgileri şu hocalara iletildi: %s' % ", ".join(okutmanlar)}


class NotGirisi(CrudView):
    """Okutman Not Girişi

    Okutmanların Sınavlara ait öğrenci notlarını sisteme girebilmesini
    sağlayan workflowa ait metdodları barındıran sınıftır.

    """

    class Meta:
        model = "DegerlendirmeNot"

    def ders_secim(self):
        """Not Girişi Ders Seçimi

        Okutmanın kendisine ait şubelerin listelendiği seçim adımına
        ait olan metot.

        Bu method seçilen şubeyi bir sonraki workflow adımı olan ve
        ``sinav_sec`` methodu ile elde edilen sınav seçim adımına
        aktarmaktadır.

        """

        _form = forms.JsonForm(current=self.current, title="Ders Seçim Formu")
        user = self.current.user
        subeler = Sube.objects.filter(okutman_id=self.get_okutman_key)
        _form.sube = fields.Integer("Sube Seçiniz",
                                    choices=prepare_choices_for_model(Sube,
                                                                      okutman_id=self.get_okutman_key))
        _form.sec = fields.Button("Seç", cmd="Ders Şubesi Seçin")
        self.form_out(_form)

    def sinav_sec(self):
        """Not Girişi Sınav Seç

        Okutmanın, seçilen şubeye ait sınavları görebildiği ve sınav
        seçimi yapabildiği adıma ait olan method.

        Seçilen şube, ``ders_secim`` methodu içinde tanımlanmış olan
        ``JsonForm`` tarafından iletilmekte ve bu method içerisinde
        aşağıdaki şekilde elde edilmektedir::

            sube_key = self.current.input['form']['sube']

        Sinavlar, bir önceki şube seçim adımında seçilmiş olan şubeye
        ait olacak şekilde filtrelenmektedir::

            _form.sinav=fields.Integer("Sınav Seçiniz",
              choices=prepare_choices_for_model(Sinav, sube_id=sube_key))

        """

        _form = forms.JsonForm(current=self.current, title="Sınav Seçim Formu")

        try:
            sube_key = self.current.input['form']['sube']
        except:
            sube_key = self.current.task_data["sube"]

        _form.sinav = fields.Integer("Sınav Seçiniz",
                                     choices=prepare_choices_for_model(Sinav, sube_id=sube_key))
        self.current.task_data["sube"] = sube_key
        _form.sec = fields.Button("Seç", cmd="Sınav Seçin")
        self.form_out(_form)

    def sinav_kontrol(self):
        """Seçilen sınava ait notların onaylanmış (teslim edilmiş) olup
        olmadığını kontrol eden method.

        Bu method ile task_data içerisine seçili sınavın onay durumu
        aşağıdaki şekilde tanımlanmaktadır::

            self.current.task_data['sinav_degerlendirme'] = sinav.degerlendirme

        Bu tanımlama ile, BMPN dosyası üzerinde tanımladığmız XOR kapısı
        sayesinde notları onaylanmış (teslim edilmiş) sınavlara ait
        notlar bir önizleme ekranında görüntülenrek, tekrar işlenmesi
        ve değiştirilmesi engellenmektedir.

        """

        sinav_key = self.current.input['form']['sinav']
        self.current.task_data['sinav_key'] = sinav_key
        sinav = Sinav.objects.get(sinav_key)
        self.current.task_data['sinav_degerlendirme'] = sinav.degerlendirme

    def not_girisi(self):
        """Seçilen sınava ait notların sisteme girilmesini sağlayan method.
        Bu method hem ``sinav_kontrol`` hem de ``not_kontrol`` methodu
        üzerinden gelen yönlendirmeleri karşılamaktadır.

        ``not_kontrol`` methodu (not kontrol adımı), okutmanın girdiği
        notları değiştirme ihtiyacı göz önünde bulundurularak
        geliştirilmiştir.

        Not kontrol adımı üzerinden gelen yönlendirmelerde güncel not
        verisini elde edebilmek için ``task_data`` altında notlar
        tanımına bakılmaktadır. Bu tanım ``not_kontrol`` methodu
        içerisinde yapıldığı, için bu veriyi barındıran bütün
        yönlendirmelerin bu adımdan yapıldığı bilinmektedir.

        Bu tanımın bulunmadığı yönlendirmelerde ise, notlar
        veritabanı üzerinden sorgulanarak listelenmektedir.

        Bu methodun barındırdığı ``NotGirisForm`` adlı form ise
        ``not_form_inline_edit`` adlı form modifier ile
        ``degerlendirme`` ve ``aciklama`` alanlarına inline edit
        özelliği kazandırılmaktadır.

        """

        _form = NotGirisForm(current=self.current, title="Not Giriş Formu")
        sinav_key = self.current.task_data['sinav_key']
        sube_key = self.current.task_data["sube"]
        sinav = Sinav.objects.get(sinav_key)

        try:
            ogrenciler = self.current.task_data["notlar"]
            for ogr in ogrenciler:
                _form.Ogrenciler(ad_soyad='%s' % (ogr['ad_soyad']),
                                 ogrenci_no=ogr['ogrenci_no'], degerlendirme=ogr['degerlendirme'],
                                 aciklama=ogr['aciklama'],
                                 key=ogr['key'])
        except:
            ogrenciler = OgrenciDersi.objects.filter(sube_id=sube_key)

            for ogr in ogrenciler:
                try:  # Öğrencinin bu sınava ait daha önceden kayıtlı notu var mı?
                    degerlendirme = DegerlendirmeNot.objects.get(sinav=sinav,
                                                                 ogrenci=ogr.ogrenci_program.ogrenci)
                    puan = degerlendirme.puan
                    aciklama = degerlendirme.aciklama
                    degerlendirme_key = degerlendirme.key
                except:
                    puan = 0
                    aciklama = ""
                    degerlendirme_key = ""

                _form.Ogrenciler(ad_soyad='%s %s' % (
                ogr.ogrenci_program.ogrenci.ad, ogr.ogrenci_program.ogrenci.soyad),
                                 ogrenci_no=ogr.ogrenci_program.ogrenci_no, degerlendirme=puan,
                                 aciklama=aciklama,
                                 key=degerlendirme_key)

        _form.kaydet = fields.Button("Önizleme", cmd="not_kontrol")
        self.form_out(_form)
        self.current.output["meta"]["allow_actions"] = False
        self.current.output["meta"]["allow_add_listnode"] = False

    def not_kontrol(self):
        """Okutmanların girmiş olduğu öğrenci notlarının listelenmesini
        sağlayan method.

        Bu method hem ``sinav_kontrol`` methodu hem de ``not_girisi``
        methodları üzerinden yapılan yönlendirmeleri karşılar. Eğer
        seçilen sınava ait notlar onaylanmış (teslim edilmiş) ise
        ``not_giris`` methoduna başlı olan not giriş adımı atlanarak bu
        adıma yönlendirme yapılmaktadır. Bu durumda notlar, veritabanı
        üzerinden alınarak listelenirken, ``not_girisi`` adımından gelen
        yönlendirmelerde form ile birlikte gönderilen veriler
        listelenmektedir.

        ``sinav_kontrol`` methodu ile yapılan yönlendirmelerde notlar
        üzerinde herhangi bir güncelleme, değişiklik yapılamayacağı için
        bu operasyonlara ait form düğmeleri gösterilmemekte ve okutmana
        bu dersler üzerinde bir değişiklik yapamayacağını bildiren bir
        mesaj kutusu gösterilmektedir.

        """

        _form = forms.JsonForm(current=self.current, title="Not Önizleme Ekranı")

        try:  # Eğer istek sinav_kontrol aşamasından yönlendirilmemişse öğrenci notları için formdan gelen veriyi kullan
            ogrenci_notlar = self.current.input['form']['Ogrenciler']
            self.current.task_data["notlar"] = ogrenci_notlar

            notlar = []
            for ogr in ogrenci_notlar:
                ogrnot = OrderedDict({})
                ogrnot['Öğrenci No'] = ogr['ogrenci_no']
                ogrnot['Adı Soyadı'] = ogr['ad_soyad']
                ogrnot['Değerlendirme'] = ogr['degerlendirme']
                ogrnot['Açıklama'] = ogr['aciklama']
                notlar.append(ogrnot)

        except:  # Eğer istek sinav_kontrol aşamasından yönlendirilmişse notlar için veritabanı kayıtlarını kullan
            sinav_key = self.current.task_data['sinav_key']
            sube_key = self.current.task_data["sube"]
            sinav = Sinav.objects.get(sinav_key)
            ogrenciler = OgrenciDersi.objects.filter(sube_id=sube_key)
            notlar = []

            for ogr in ogrenciler:
                try:  # Öğrencinin bu sınava ait daha önceden kayıtlı notu var mı?
                    degerlendirme = DegerlendirmeNot.objects.get(sinav=sinav,
                                                                 ogrenci=ogr.ogrenci_program.ogrenci)
                    puan = degerlendirme.puan
                    aciklama = degerlendirme.aciklama
                except:
                    puan = 0
                    aciklama = ""

                ogrnot = OrderedDict({})
                ogrnot['Öğrenci No'] = ogr.ogrenci_program.ogrenci_no
                ogrnot['Adı Soyadı'] = '%s %s' % (
                ogr.ogrenci_program.ogrenci.ad, ogr.ogrenci_program.ogrenci.soyad)
                ogrnot['Değerlendirme'] = puan
                ogrnot['Açıklama'] = aciklama
                notlar.append(ogrnot)

        # Eğer notlar okutman tarından onaylanmışsa (teslim edilmişse) uyarı göster
        if self.current.task_data['sinav_degerlendirme']:
            self.current.output['msgbox'] = {

                'type': 'info', "title": 'Notlar Onaylandı',
                "msg": 'Bu derse ait notlar onaylanmış olduğu için içeriği değiştirilemez.'

            }
            _form.ders_secim = fields.Button("Ders Seçim Ekranına Dön", cmd="ders_sec",
                                             flow="ders_secim_adimina_don")
            _form.sinav_secim = fields.Button("Sınav Seçim Ekranına Dön", cmd="sinav_sec",
                                              flow="sinav_secim_adimina_don")

        else:  # Eğer notlar hala onaylanmamışsa (teslim edilmemişse) form düğmelerini göster

            _form.not_onay = fields.Boolean("Sınav Notlarını Onaylıyorum (Bu işlem geri alınamaz!)")
            _form.not_duzenle = fields.Button("Notları Düzenle", cmd="not_girisi",
                                              flow="not_giris_formuna_don")
            _form.kaydet = fields.Button("Kaydet", cmd="not_kaydet", flow="end")
            _form.kaydet_ve_ders_sec = fields.Button("Kaydet ve Ders Seçim Ekranına Dön",
                                                     cmd="ders_sec",
                                                     flow="ders_adimina_don")
            _form.kaydet_ve_sinav_sec = fields.Button("Kaydet ve Sınav Seçim Ekranına Dön",
                                                      cmd="sinav_sec",
                                                      flow="sinav_adimina_don")

        self.current.output['object'] = {
            "type": "table-multiRow",
            "fields": notlar,
            "actions": False
        }
        self.form_out(_form)

    def not_kaydet(self):
        """Okutmanın girmiş olduğu notların veritabanına kaydedilmesini
        sağlayan method.

        Bu method, önceden girilmiş olan notları veritabanı üzerinde
        güncellerken, key verisi olmayan not girişleri için veritabanı
        üzerinde yeni bir kayıt açmaktadır.

        """

        term = Donem.objects.filter(guncel=True)[0]
        sinav_key = self.current.task_data["sinav_key"]
        sube_key = self.current.task_data["sube"]

        sinav = Sinav.objects.get(sinav_key)
        ders = sinav.ders

        for ogrenci_not in self.current.task_data["notlar"]:

            try:
                ogr_data = OgrenciProgram.objects.get(ogrenci_no=ogrenci_not['ogrenci_no'])

                if ogrenci_not['key']:  # Önceden girilmiş bir kayıt mı?
                    ogr_not = DegerlendirmeNot.objects.get(ogrenci_not['key'])
                else:
                    ogr_not = DegerlendirmeNot()

                ogr_not.puan = ogrenci_not['degerlendirme']
                ogr_not.aciklama = ogrenci_not['aciklama']
                ogr_not.ogrenci_no = ogrenci_not['ogrenci_no']
                ogr_not.donem = '%s' % term.ad
                ogr_not.yil = term.baslangic_tarihi.year
                ogr_not.ogretim_elemani = self.get_okutman_name_surname
                ogr_not.ders = ders
                ogr_not.sinav = sinav
                ogr_not.ogrenci = ogr_data.ogrenci
                ogr_not.sinav_tarihi = sinav.tarih
                ogr_not.save()

            except:
                pass

        # Okutman notları onayladığını (teslim ettiğini) bildirmişse

        if self.current.input['form']['not_onay']:
            sinav.degerlendirme = True
            sinav.save()

    def kayit_bilgisi_ver(self):
        sinav_key = self.current.task_data["sinav_key"]
        sinav = Sinav.objects.get(sinav_key)
        sinav_tarihi = sinav.tarih.strftime("%d/%m/%Y")

        self.current.output['msgbox'] = {
            'type': 'info', "title": 'Notlar Kaydedildi',
            "msg": '%s dersine ait %s tarihli sınav notları kaydedildi' % (
            sinav.ders.ad, sinav_tarihi)}

    @property
    def get_okutman_key(self):
        """Harici okutman ve okutman kayıt key'lerinin ayrımını sağlayan method.
        """
        return self.current.user.personel.okutman.key if self.current.user.personel.key else self.current.user.harici_okutman.okutman.key

    @property
    def get_okutman_name_surname(self):
        """Harici okutman ve okutman ad,soyad ayrımını sağlayan method.
        """
        return "%s %s" % (self.current.user.personel.okutman.ad,
                          self.current.user.personel.okutman.soyad) if self.current.user.personel.key else "%s %s" % (
            self.current.user.harici_okutman.ad, self.current.user.harici_okutman.soyad)



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Nufus Guncelle

Hitap'a personelin Nufus bilgilerinin guncellenmesini yapar.

"""

__author__ = 'H.İbrahim Yılmaz (drlinux)'

from ulakbus.services.personel.hitap.hitap_guncelle import HITAPGuncelle


class HizmetNufusGuncelle(HITAPGuncelle):
    """
    HITAP Ekleme servisinden kalıtılmış Hizmet Nufus Bilgi Guncelleme servisi

    """

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): : Request yoluyla gelen kayıtlar,
                    HizmetNufusUpdate servisinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları ve servis tarafında gerekli olan
                    alanlar listede tutulmaktadır.

        """

        self.service_name = 'HizmetNufusUpdate'
        self.service_dict = {
            'fields': {
                'ad': self.request.payload.get('ad', ''),
                'cinsiyet': self.request.payload.get('cinsiyet', ''),
                'dogumTarihi': self.request.payload.get('dogum_tarihi', ''),
                'durum': self.request.payload.get('durum', ''),
                'emekliSicilNo': self.request.payload.get('emekli_sicil_no', ''),
                'ilkSoyad': self.request.payload.get('ilk_soy_ad', ''),
                'kurumSicili': self.request.payload.get('kurum_sicil', ''),
                'maluliyetKod': self.request.payload.get('maluliyet_kod', ''),
                'memuriyetBaslamaTarihi': self.request.payload.get('memuriyet_baslama_tarihi', ''),
                'sebep': self.request.payload.get('sebep', ''),
                'soyad': self.request.payload.get('soyad', ''),
                'tckn': self.request.payload.get('tckn', ''),
                'aciklama': self.request.payload.get('aciklama', ''),
                'yetkiSeviyesi': self.request.payload.get('yetki_seviyesi', ''),
                'kurumaBaslamaTarihi': self.request.payload.get('kuruma_baslama_tarihi', ''),
                'gorevTarihi6495': self.request.payload.get('gorev_tarihi_6495', ''),
                'emekliSicil6495': self.request.payload.get('emekli_sicil_6495', '')
            },
            'date_filter': ['dogumTarihi', 'memuriyetBaslamaTarihi', 'kurumaBaslamaTarihi'],
            'required_fields': ['tckn', 'ad', 'soyad', 'dogumTarihi', 'cinsiyet', 'emekliSicilNo',
                                'memuriyetBaslamaTarihi', 'durum', 'kurumSicili', 'maluliyetKod',
                                'sebep', 'yetkiSeviyesi']

        }
        super(HizmetNufusGuncelle, self).handle()



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
from zengine.lib.cache import Cache


class RolePermissionCache(Cache):
    PREFIX = 'PERM'

    def __init__(self, role_id):
        super(RolePermissionCache, self).__init__(role_id)

#
# class Cache(Cache):
#     PREFIX = ''
#
#     overriding of init optional but helpful
#     def __init__(self, ):
#         super(, self).__init__()



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
import random
from zengine.forms import JsonForm
from zengine.forms import fields
from zengine.views.base import SimpleView, BaseView
from zengine.views.crud import CrudView


class TCKNForm(JsonForm):
    class Meta:
        title = 'Yeni Personel'

    tcno = fields.String("TC No")
    goto_hata = fields.Button("Ekleme")
    ekle = fields.Button("Ekle")


class YeniPersonelEkle(BaseView):
    def __init__(self, current=None):
        current.output['forms'] = TCKNForm().serialize()
        current.output['client_cmd'] = ['form']


def get_personel_from_hitap(current):
    tcno = current.input['form']['tcno']
    current.task_data['hitap_tamam'] = current.input['form'].get('goto_hata') != 1
    current.task_data['tcno'] = tcno
    current.set_message(title='%s TC no için Hitap servisi başlatıldı' % tcno,
                        msg='', typ=1, url="/wftoken/%s" % current.token)


def get_by_tckn(current):
    current.task_data['aks_tamam'] = True


def get_from_mernis(current):
    # %60 ihtimalle yeni atama ekranina gidecek, %40  hata verecek...
    # current.task_data['mernis_tamam'] = random.choice((1, 2, 3, 4, 5)) in (1, 2, 3)
    current.task_data['mernis_tamam'] = True
    current.set_message(title='%s için Mernis\'e erişilemedi' % current.task_data['tcno'],
                        msg='', typ=1, url="/wftoken/%s" % current.token)


class AtamaYap(CrudView):

    class Meta:
        model = 'Kadro'

    class ObjectForm(JsonForm):
        save_list = fields.Button("Kaydet", cmd="save::list", flow="goto_service")
        save_edit = fields.Button("Kaydet ve Devam Et", cmd="save::devam_et", flow="goto_service")


class HataIncele(JsonForm):
    class Meta:
        title = 'İşlem Başarısız Oldu'
        help_text = "Dasd askdhjkasbdajs gasjkhdgasjkhg ajhdgas askjdhaskjhasj dhas dhaskjdhas" \
                    "aklsjsd haskljdhasklj dhaskldh akdhaksj dajskh dgasjkhdg ajshgdasj dgas" \
                    "aslkd haskldhaskdhaskldhaskl dhaklsd"

    restart = fields.Button("Tekrar Dene", flow="hata_to_tcno")
    cancel = fields.Button("İşlemi İptal Et", flow="iptal_hata")


# class Iptal(JsonForm):
#     class Meta:
#         title = 'İptal'
#         help_text = "Başarıyla iptal edildi"

def delete_draft(current):
    current.output['msgbox'] = {'type': 'info', "title": 'İptal', "msg": 'Başarıyla iptal edildi'}


class review_service_errors(SimpleView):
    def show_view(self):
        self.output['forms'] = HataIncele(current=self.current).serialize()


def yeni_ekle_kontrol(current):
    current.output['object'] = {'bumudur': 'budur'}
    current.output['client_cmd'] = ['show']



# -*-  coding: utf-8 -*-
"""Öğrenci ile ilgili yardımcı class ve fonksiyonların yer aldığı dosyadır.
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

__author__ = 'H.İbrahim Yılmaz (drlinux)'

from ulakbus.models import Ogrenci, OgrenciProgram, OgrenciDersi
from pyoko.exceptions import ObjectDoesNotExist


class OgrenciHelper():
    """OgrenciHelper Class

    """

    def diploma_notu_uret(self, ogrenci_no):
        try:
            ogrenci_program = OgrenciProgram.objects.get(ogrenci_no=ogrenci_no)
            return "%s-%s-%s" % (ogrenci_program.giris_tarihi,
                                  ogrenci_program.program.yoksis_no, ogrenci_program.ogrenci_no)
        except ObjectDoesNotExist:
            return "Öğrenci Bulunamadı"



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Cetvel Ekle

Hitap'a personelin Hizmet Cetvel  Kayit bilgilerinin eklenmesini yapar.

"""

__author__ = 'H.İbrahim Yılmaz (drlinux)'

from ulakbus.services.personel.hitap.hitap_ekle import HITAPEkle


class HizmetCetveliEkle(HITAPEkle):
    """
    HITAP Ekleme servisinden kalıtılmış Hizmet Okul Bilgi Ekleme servisi

    """

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): Request yoluyla gelen kayıtlar,
                    HizmetCetvelInsert servisinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları listede tutulmaktadır.
        """

        self.service_name = 'HizmetCetvelInsert'
        self.service_dict = {
            'fields': {
                'baslamaTarihi': self.request.payload.get('baslama_tarihi', ''),
                'bitisTarihi': self.request.payload.get('bitis_tarihi', ''),
                'emekliDerece': self.request.payload.get('emekli_derece', ''),
                'emekliKademe': self.request.payload.get('emekli_kademe', ''),
                'gorev': self.request.payload.get('gorev', ''),
                'unvanKod': self.request.payload.get('unvan_kod', ''),
                'hizmetSinifi': self.request.payload.get('hizmet_sinifi', ''),
                'kazanilmisHakAyligiDerece': self.request.payload.get(
                    'kazanilmis_hak_ayligi_derece', ''),
                'kazanilmisHakAyligiKademe': self.request.payload.get(
                    'kazanilmis_hak_ayligi_kademe', ''),
                'odemeDerece': self.request.payload.get('odeme_derece', ''),
                'odemeKademe': self.request.payload.get('odeme_kademe', ''),
                'emekliEkGosterge': self.request.payload.get('emekli_ekgosterge', ''),
                'kadroDerece': self.request.payload.get('kadro_derece', ''),
                'kazanilmisHakAyligiEkGosterge': self.request.payload.get(
                    'kazanilmis_hak_ayligi_ekgosterge', ''),
                'odemeEkGosterge': self.request.payload.get('odeme_ekgosterge', ''),
                'sebepKod': self.request.payload.get('sebep_kod', ''),
                'tckn': self.request.payload.get('tckn', ''),
                'ucret': self.request.payload.get('ucret', ''),
                'yevmiye': self.request.payload.get('yevmiye', ''),
                'kurumOnayTarihi': self.request.payload.get('kurum_onay_tarihi', '')
            },
            'date_filter': ['baslamaTarihi', 'bitisTarihi', 'kurumOnayTarihi'],
            'required_fields': ['tckn', 'gorev', 'unvanKod', 'hizmetSinifi', 'kadroDerece',
                                'odemeDerece', 'odemeKademe', 'odemeKademe', 'odemeEkGosterge',
                                'kazanilmisHakAyligiDerece', 'kazanilmisHakAyligiKademe',
                                'kazanilmisHakAyligiEkGosterge', 'emekliDerece', 'emekliKademe',
                                'emekliEkGosterge', 'sebepKod', 'kurumOnayTarihi']
        }
        super(HizmetCetveliEkle, self).handle()



# -*-  coding: utf-8 -*-
from zengine import forms
from zengine.forms import fields
from zengine.views.crud import CrudView
from ulakbus.models.ogrenci import Ogrenci, OgrenciProgram, OncekiEgitimBilgisi


class YerlestirmeBilgisiForm(forms.JsonForm):
    class Meta:
        include = ["giris_puan_turu", "giris_puani"]

    ileri_buton = fields.Button("İleri", cmd="save")


class YerlestirmeBilgisi(CrudView):
    class Meta:
        model = "OgrenciProgram"

    def yerlestirme_bilgisi_form(self):
        ogrenci = Ogrenci.objects.get(user=self.current.user)
        ogrenci_program = OgrenciProgram.objects.get(ogrenci=ogrenci, durum=1)
        self.form_out(YerlestirmeBilgisiForm(ogrenci_program, current=self.current))


class OncekiEgitimBilgileriForm(forms.JsonForm):
    class Meta:
        include = ["okul_adi", "diploma_notu", "mezuniyet_yili"]

    kaydet = fields.Button("Kaydet", cmd="save")


class OncekiEgitimBilgileri(CrudView):
    class Meta:
        model = "OncekiEgitimBilgisi"

    def onceki_egitim_bilgileri(self):
        ogrenci = Ogrenci.objects.get(user=self.current.user)
        onceki_egitim_bilgisi = OncekiEgitimBilgisi.objects.filter(ogrenci=ogrenci)
        self.form_out(OncekiEgitimBilgileriForm(onceki_egitim_bilgisi[0], current=self.current))

    def kaydet(self):
        ogrenci = Ogrenci.objects.get(user=self.current.user)
        self.set_form_data_to_object()
        self.object.ogrenci = ogrenci
        self.object.save()


class OnKayitForm(forms.JsonForm):
    class Meta:
        title = "Kişisel Bilgiler"
        include = ['kan_grubu', 'baba_aylik_kazanc', 'baba_ogrenim_durumu', 'baba_meslek',
                   'anne_ogrenim_durumu', 'anne_meslek', 'anne_aylik_kazanc', 'masraf_sponsor',
                   'emeklilik_durumu', 'kiz_kardes_sayisi', 'erkek_kardes_sayisi',
                   'ogrenim_goren_kardes_sayisi', 'burs_kredi_no', 'aile_tel', 'aile_gsm',
                   'aile_adres', 'ozur_durumu', 'ozur_oran']

    kaydet_buton = fields.Button("Kaydet", cmd="kaydet")


class OnKayit(CrudView):
    class Meta:
        model = "Ogrenci"

    def on_kayit_form(self):
        ogrenci = Ogrenci.objects.get(user=self.current.user)
        self.form_out(OnKayitForm(ogrenci, current=self.current))


class BelgeForm(forms.JsonForm):
    class Meta:
        include = ["Belgeler"]

    kaydet = fields.Button("Kaydet", cmd="save")
    onayla = fields.Button("Ön Kayıt Onayla", cmd="onayla")


class KayitBelgeler(CrudView):
    class Meta:
        model = "OgrenciProgram"

    def belge_form(self):
        ogrenci = Ogrenci.objects.get(user=self.current.user)
        ogrenci_program = OgrenciProgram.objects.get(ogrenci=ogrenci)
        self.form_out(BelgeForm(ogrenci_program, current=self.current))

    def onayla(self):
        ogrenci = Ogrenci.objects.get(user=self.current.user)
        ogrenci_program = OgrenciProgram.objects.get(ogrenci=ogrenci, durum=1)
        ogrenci_program.durum = 2
        ogrenci_program.save()
